<?php
namespace Teknisa\PLA\Modules\Medicao\Controllers;

use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Pla01000EfetivosController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $parameterBagColumns = array();
    protected $environment;
    protected $entityManager;
    protected $efetivosService;
    protected $hasRetirada = array();
    protected $dataSourceNameToSave;
    protected $connection;
    protected $dataSourceManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager, $efetivosService, $dataSourceManager) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->efetivosService   = $efetivosService;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection        = $entityManager->getConnection();
    }

    public function getParams($request, $response) {
        try {
            $row = $request->getRow();
            $paramgeral = $this->crudService->getFilteredDataSet("paramgeral", array());
            $conditions = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $row["CDFILIAL"]
                )
            );
            $paramplaneja = $this->crudService->getFilteredDataSet("paramplaneja", $conditions);
            $retiradas = $this->getRetiradas($row["SERVICOS"]);
           
            $paramfilial = $this->getDtProcessa($row["CDFILIAL"]);
            $response->addDataSet($paramgeral);
            $response->addDataSet($paramplaneja);
            $response->addDataSet($retiradas);
            $response->addDataSet($paramfilial);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
     public function getDtProcessa ($cdfilial){
        $queryDtProcessa = "SELECT CASE WHEN IDUTILDTPROC = 'S' THEN NVL(PF.DTPROCESSA, SYSDATE) ELSE SYSDATE END AS DTPROCESSA
                              FROM PARAMFILIAL PF INNER JOIN PARAMGERAL PG ON PG.CDPARGERAL = '0001' AND PF.CDFILIAL = :P_CDFILIAL";
        $stmt = $this->connection->prepare($queryDtProcessa);
        $stmt->execute(array(
            "P_CDFILIAL"   => $cdfilial
        ));
        return new DataSet("paramfilial", $stmt->fetchAll(\PDO::FETCH_ASSOC));
    }
    
    public function verificaRetirada (Request\Row $request, Response $response){
        $queryRetirada = "SELECT
                   SUM(NRCARDAPIO) AS CARDAPIO
                FROM
                   RETIRADA
                WHERE
                   (CDFILIAL   = :CDFILIAL) AND
                   (NRCARDAPIO = :NRCARDAPIO)";
        $row = $request->getRow();
        $stmt = $this->connection->prepare($queryRetirada);
        $stmt->execute(array(
            "CDFILIAL"   => $row["CDFILIAL"],
            "NRCARDAPIO" => $row["NRCARDAPIO"]
        ));
        $response->addDataSet(new DataSet('RETIRADA', $stmt->fetchAll(\PDO::FETCH_ASSOC)));
    }
    
    public function verificaEfetivosJaFat ($CDFILIAL, $CDSERVFILI, $CDSERVPLAN, $CDCLIENTE, $DTEFETCLIE){
        $sql = "SELECT CDFILIAL
                FROM EFETCLIE
                WHERE CDFILIAL = :CDFILIAL
                  AND CDSERVFILI = :CDSERVFILI
                  AND CDSERVPLAN = :CDSERVPLAN
                  AND CDCLIENTE = :CDCLIENTE
                  AND DTEFETCLIE = :DTEFETCLIE
                  AND NRAPURACFAT IS NOT NULL";
        
       
        
            $stmt = $this->connection->prepare($sql);
            $result = $stmt->execute(array(
                "CDFILIAL"   => $CDFILIAL,
                "CDSERVFILI" => $CDSERVFILI,
                "CDSERVPLAN" => $CDSERVPLAN,
                "CDCLIENTE" => $CDCLIENTE,
                "DTEFETCLIE" => $DTEFETCLIE
            ));
            
            $result = $stmt->fetchAll();
            
            return count($result) > 0;
    }
    
    function verificaRetiradaClientesAFaturar (Request\Row $request, Response $response){
        
        $this->verificaRetirada($request, $response);
        
        $row = $request->getRow();
        
        $sql = "SELECT S.CDFILIAL
                  FROM SERVCLIE S 
                 WHERE S.CDFILIAL = :P_CDFILIAL
                   AND S.CDSERVFILI IN (
                       SELECT CDSERVICFATURAM FROM SERVIFAT WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI
                        UNION
                       SELECT :P_CDSERVFILI FROM DUAL)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            "P_CDFILIAL"   => $row["CDFILIAL"],
            "P_CDSERVFILI" => $row["CDSERVFILI"]
        ));
        $result = $stmt->fetchAll();
        
        if(count($result) == 0){
            $this->efetivosService->saveCardadia($row);
            $response->addDataSet(new DataSet('SEM_CLIENTES_SERVICO', [true]));
        }else {
            $sql = "SELECT S.CDFILIAL
                      FROM SERVCLIE S
                           LEFT JOIN EFETCLIE E ON E.CDFILIAL = S.CDFILIAL AND
                                                   E.CDCLIENTE = S.CDCLIENTE AND
                                                   E.DTEFETCLIE = :P_DTCARDAPIO
                     WHERE S.CDFILIAL = :P_CDFILIAL
                       AND S.CDSERVFILI IN (SELECT CDSERVICFATURAM FROM SERVIFAT WHERE CDFILIAL = :P_CDFILIAL2 AND CDSERVFILI = :P_CDSERVFILI
                                             UNION
                                            SELECT :P_CDSERVFILI2 FROM DUAL)
                       AND (E.CDFILIAL IS NULL OR E.NRAPURACFAT IS NULL)
                       AND ROWNUM = 1
                    ";
            $stmt = $this->connection->prepare($sql);
            $stmt->execute(array(
                "P_DTCARDAPIO" => substr($row["DTCARDAPIO"], 0, 10),
                "P_CDFILIAL"   => $row["CDFILIAL"],
                "P_CDFILIAL2"   => $row["CDFILIAL"],
                "P_CDSERVFILI" => $row["CDSERVFILI"],
                "P_CDSERVFILI2" => $row["CDSERVFILI"]
            ));
            $result = $stmt->fetchAll();
            if(count($result) == 0){
                $response->addDataSet(new DataSet('SEM_CLIENTES_A_FATURAR', [true]));
            }
        }
    }
    
    public function getRetiradas($servicos) {
        $retiradas = array();
        foreach ($servicos as $servico) {
            $cdfilial = $servico["CDFILIAL"];
            $cdservfili = $servico["CDSERVFILI"];
            $nrcardapio = $servico["NRCARDAPIO"];
            $retirada = $this->efetivosService->getRetirada($cdfilial, $nrcardapio);
            if (empty($retirada)) {
                $retiradas[$cdservfili]  =  false;
            } else {
                $retiradas[$cdservfili]  =  true;
            }
        }
        return new DataSet("retiradas", array($retiradas));
    }

    public function saveComensais(Request\Row $request, Response $response) {
        try {
            $this->connection->beginTransaction();
            $row = $request->getRow();
            $comPla = $row["COMPLA"];
            $comFat = $row["COMFAT"];
            $this->efetivosService->saveCardadia($comPla);
            $clientesJaFat = array();
            foreach ($comFat as $cliente) {
                if ($cliente["CDCLIENTE"]) {
                    $jaFat = false;
                    if( isset($cliente["QTCOMEFETCLI_inline"]) || isset($cliente["QTVENDAEFETCLI_inline"]) ){
                        $jaFat = $this->verificaEfetivosJaFat($cliente['CDFILIAL'], $cliente['CDSERVFILI'], $cliente['CDSERVPLAN'], $cliente['CDCLIENTE'], substr($cliente['DTEFETCLIE'], 0, 10));
                        if($jaFat){
                            $clientesJaFat[] = $cliente['NMRAZSOCCLIE'];
                        }
                    }
                    if(!$jaFat)
                        $this->efetivosService->saveEfetclie($cliente, $request, $response);
                }
            }
            if(count($clientesJaFat) > 0){
                $this->connection->rollback();
                $response->addDataSet(new DataSet('ErroClientesJaFatNAlterar', $clientesJaFat));
            }
            else
                $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }

    public function saveTurno(Request\DataSet $request, Response $response) {
        try {
            $this->connection->beginTransaction();
            $turnos = $request->getDataSet()->getRows();
            foreach ($turnos as $turno) {
                $this->efetivosService->saveTurnoclie($turno);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    public function verificaTurnos($request, $response) {
        try {
            $row = $request->getRow();
            $dataSet = array();
            foreach($row['CDCLIENTE'] as $cliente) {
                $conditions = array(
                    array (
                        "columnName" => "CDFILIAL",
                        "operator" => "=",
                        "value" => $row["CDFILIAL"]
                    ),
                    array (
                        "columnName" => "CDSERVFILI",
                        "operator" => "=",
                        "value" => $row["CDSERVFILI"]
                    ),
                    array (
                        "columnName" => "DTDATA",
                        "operator" => "=",
                        "value" => $row["DTDATA"]
                    ),
                    array (
                        "columnName" => "CDCLIENTE",
                        "operator" => "=",
                        "value" => $cliente
                    )
                );
                $turnos = $this->crudService->getFilteredDataSet("pla01000_turno_quantidade", $conditions);
                foreach($turnos->getRows() as $turno){
                    $dataSet[] = $turno;
                }
            }
            $dataSet = new DataSet('pla01000_verificaTurnos', $dataSet);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    public function salvaQuantidadeProduzida(Request\Row $request, Response $response) {
        try {
            $this->connection->beginTransaction();
            $row = $request->getRow();
            if(isset($row['modificados'])) {
                $alterados=array();
                foreach($row['modificados'] as $mod) {
                    $this->efetivosService->saveQtdeProduzidaCardadia($mod);
                    $alterados[] = $mod;
                }
                $dataSet = new DataSet('salvaQuantidadeProduzida', $alterados);
                $response->addDataSet($dataSet);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
}