<?php
namespace Teknisa\PLA\Modules\Medicao\Controllers;

use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\PebLibs\Services\ValidacoesEstoqueService;
use \Estoque\Entities\Lanctoestoq;

class Pla08000RetiradaGeralController {

    protected $newCode;
    protected $connection;
    protected $util;
    protected $environment;
    protected $libUtilities;

    public function __construct($newCode, $connection, $util, $environment, ValidacoesEstoqueService $validacoesEstoque,$libUtilities) {
        $this->newCode = $newCode;
        $this->connection = $connection;
        $this->util = $util;
        $this->environment = $environment;
        $this->validacoesEstoque = $validacoesEstoque;
        $this->libUtilities  = $libUtilities;
    }

    public function realizaCommit() {
        if($this->connection->isTransactionActive())
            $this->connection->commit();
    }

    public function realizaRollback() {
        if($this->connection->isTransactionActive())
            $this->connection->rollback();
    }
    
    /* INICIO DE INCLUSAO DE UM SERVICO */

    public function retiradaAutomatica(Request $request, Response $response) {
        $row = $request->getRow();
         //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL_FILTRADA'],'RETIRADA');
        try {
            $row['RETIRADAAUT'] = true;
            $this->verificaQuantidadeRetirada($row);
            $this->percorreServicos($row, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function verificaQuantidadeRetirada( $row ) {
        $query = "SELECT NRRETGERAL FROM RETIRADAGERAL RE WHERE RE.CDFILIAL = :CDFILIAL AND TRUNC(DTRETGERAL) = TO_DATE(:DTRETGERAL, 'DD/MM/YYYY') AND ( ( INSTR(:P_SERVFILI, RE.CDSERVFILI) > 0 ) OR ( :P_SERVFILI = 'T' ) ) AND EXISTS ( SELECT 1 FROM SERVICO S WHERE S.CDFILIAL = RE.CDFILIAL AND S.CDSERVFILI = RE.CDSERVFILI AND ( ( INSTR(:P_GRUPSERV, S.CDGRUPSERV) > 0 ) OR ( :P_GRUPSERV = 'T' ) ) )";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL"   => $row['CDFILIAL_FILTRADA'],
            "DTRETGERAL" => $row['DTRETGERAL'],
            "P_SERVFILI" => $row['NMSERVICO'],
            "P_GRUPSERV" => $row['NMGRUPSERV']
        );
        $stmt->execute($params);
        $result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(empty($result)) {
            throw new \Exception("Operação bloqueada. Não há nenhum produto com quantidade consumida.");
        }
    }
    
    public function verificaRetirada(Request $request, Response $response) {
        $row = $request->getRow();
        try {
            $this->existeRetiradaAutomatica($row, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function existeRetiradaAutomatica($row, $response) {
        $query = "SELECT DISTINCT CR.CDSERVFILI, SR.NMSERVICO FROM ITEMRETI      IR, RETIRADA      RT, CARDADIA      CR, SERVICO       SR, RETIRADAGERAL R WHERE CR.CDFILIAL = :CDFILIAL AND R.CDFILIAL = CR.CDFILIAL AND R.CDSERVFILI = CR.CDSERVFILI AND ( IR.CDFILIAL = RT.CDFILIAL ) AND ( IR.NRCARDAPIO = RT.NRCARDAPIO ) AND ( IR.NRRETIRADA = RT.NRRETIRADA ) AND ( IR.QTRETIRADA > 0 ) AND ( CR.DTCARDAPIO = :DTRETIRADA ) AND ( CR.CDFILIAL = RT.CDFILIAL ) AND ( CR.NRCARDAPIO = RT.NRCARDAPIO ) AND ( CR.CDFILIAL = SR.CDFILIAL ) AND ( CR.CDSERVFILI = SR.CDSERVFILI ) AND ( R.CDFILIAL = :CDFILIAL ) AND R.DTRETGERAL = :DTRETIRADA AND ( ( INSTR(:P_SERVFILI, R.CDSERVFILI) > 0 ) OR ( :P_SERVFILI = 'T' ) ) AND ( ( INSTR(:P_GRUPSERV, SR.CDGRUPSERV) > 0 ) OR ( :P_GRUPSERV = 'T' ) ) ORDER BY 1";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL"   => $row['CDFILIAL_FILTRADA'],
            "DTRETIRADA" => $row['DTRETGERAL'],
            "P_SERVFILI" => $row['NMSERVICO'],
            "P_GRUPSERV" => $row['NMGRUPSERV']
        );
        $stmt->execute($params);
        $retiradaAutomatica = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(!empty($retiradaAutomatica)) {
            $retirada = array(
                "RETIRADA"       => true,
                "SERVICES_ARRAY" => $retiradaAutomatica
            );
            $response->addDataSet(new DataSet("dataRetirada", $retirada));
        }else{
            $retirada = array(
                "RETIRADA" => false
            );
            $response->addDataSet(new DataSet("dataRetirada", $retirada));
        }
    }
    
    private function percorreServicos($row, $response) {
        $errorProducts = array();
        
        //retorna a quantidade consumida total por produto e almoxarifado
        $query = "SELECT CDFILIAL, CDPRODUTO, DTRETGERAL, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, SUM(QTRETIRADA) AS QTRETIRADA FROM ( SELECT R.CDFILIAL, R.CDPRODUTO, R.DTRETGERAL, R.CDALMOXARIFE, R.CDLOCALESTOQ, R.NRLOTEESTQ, R.NRSUBLOTE, R.QTCONRETIGER AS QTRETIRADA FROM RETIRADAGERAL R LEFT JOIN CARDADIA      CA ON CA.CDFILIAL = R.CDFILIAL AND CA.CDSERVFILI = R.CDSERVFILI AND CA.DTCARDAPIO = :DTRETIRADA LEFT JOIN RETIRADA      RE ON RE.CDFILIAL = CA.CDFILIAL AND RE.NRCARDAPIO = CA.NRCARDAPIO AND RE.DTRETIRADA = CA.DTCARDAPIO WHERE R.CDFILIAL = :CDFILIAL AND R.DTRETGERAL = :DTRETIRADA AND R.QTCONRETIGER > 0 AND ( ( INSTR(:P_SERVFILI, R.CDSERVFILI) > 0 ) OR ( :P_SERVFILI = 'T' ) ) AND EXISTS ( SELECT 1 FROM SERVICO S WHERE S.CDFILIAL = R.CDFILIAL AND S.CDSERVFILI = R.CDSERVFILI AND ( ( INSTR(:P_GRUPSERV, S.CDGRUPSERV) > 0 ) OR ( :P_GRUPSERV = 'T' ) ) ) GROUP BY R.CDFILIAL, R.CDSERVFILI, R.CDPRODUTO, R.DTRETGERAL, R.CDALMOXARIFE, R.CDLOCALESTOQ, R.NRLOTEESTQ, R.NRSUBLOTE, R.QTCONRETIGER ) GROUP BY CDFILIAL, CDPRODUTO, DTRETGERAL, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL" => $row['CDFILIAL_FILTRADA'],
            "DTRETIRADA" => $row['DTRETGERAL'],
            "P_SERVFILI" => $row['NMSERVICO'],
            "P_GRUPSERV" => $row['NMGRUPSERV']
        );

        $stmt->execute($params);
        $sumQtdProdAlmox = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        $lancamentos = array(); // KEY: NRLANCESTQ, VALUE: SERVICE
        
        if(!empty($sumQtdProdAlmox)) {
            $errorProducts = array();
            $errorMsg = "";
            foreach ($sumQtdProdAlmox as $produto){
                $this->verificaEstoqueNegativo($produto, $response, $errorProducts, $errorMsg);
            }
            if(!empty($errorProducts)) {
                $response->addDataSet(new DataSet("produtosInvalidos", $errorProducts));
                $response->addDataSet(new DataSet("errorMsg", array($errorMsg)));
                throw new \Exception($errorMsg);
            }
            $servicoFalho = array(
                "SERVICO_FALHO" => array()
            );
            $queryServico = "SELECT R.CDFILIAL, R.CDSERVFILI, S.NMSERVICO, R.DTRETGERAL, S.IDUTRETIPRAT, MAX(CA.NRCARDAPIO) AS NRCARDAPIO, MAX(RE.NRRETIRADA) NRRETIRADA FROM RETIRADAGERAL R LEFT JOIN CARDADIA      CA ON CA.CDFILIAL = R.CDFILIAL AND CA.CDSERVFILI = R.CDSERVFILI AND CA.DTCARDAPIO = :DTRETIRADA LEFT JOIN RETIRADA      RE ON RE.CDFILIAL = CA.CDFILIAL AND RE.NRCARDAPIO = CA.NRCARDAPIO AND RE.DTRETIRADA = CA.DTCARDAPIO LEFT JOIN SERVICO       S ON S.CDFILIAL = R.CDFILIAL AND S.CDSERVFILI = R.CDSERVFILI WHERE R.CDFILIAL = :CDFILIAL AND R.DTRETGERAL = :DTRETIRADA AND R.QTCONRETIGER > 0 AND ( ( INSTR(:P_SERVFILI, R.CDSERVFILI) > 0 ) OR ( :P_SERVFILI = 'T' ) ) AND ( ( INSTR(:P_GRUPSERV, S.CDGRUPSERV) > 0 ) OR ( :P_GRUPSERV = 'T' ) ) GROUP BY R.CDFILIAL, R.CDSERVFILI, S.NMSERVICO, R.DTRETGERAL, S.IDUTRETIPRAT";
            $stmt = $this->connection->prepare($queryServico);
            $paramsServico = array(
                "CDFILIAL"   => $row['CDFILIAL_FILTRADA'],
                "DTRETIRADA" => $row['DTRETGERAL'],
                "P_SERVFILI" => $row['NMSERVICO'],
                "P_GRUPSERV" => $row['NMGRUPSERV']
            );
    
            $stmt->execute($paramsServico);
            $servicos = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            $servicosComRetirada = array();
            $ERRO_ATUALIZA_ESTOQUE = array();
            foreach ($servicos as $servico) {
                if($servico['NRRETIRADA'] == NULL) {
                    $seqItem = '0000';
                    $this->connection->beginTransaction();
                    try {
                        
                        if($servico['NRCARDAPIO'] == NULL) {
                            $novoCodigoCardapio   = $this->geraNovoCodigo('CARDAPIO'.$row['CDFILIAL_FILTRADA']);
                            $this->insereCardapio($row, $novoCodigoCardapio, $servico);
                        } else {
                            $novoCodigoCardapio = $servico['NRCARDAPIO'];
                        }
                        
                        $novoCodigoLancamento = $this->geraNovoCodigo('LANCTOEST'.$this->environment->getCdFilial());
                        $novoCodigoRetirada   = $this->geraNovoCodigo('RETIRADA'.$row['CDFILIAL_FILTRADA']);
                       
                        $idutretiprat = $servico['IDUTRETIPRAT'];
                        
                        $this->insereLancamento($row, $servico, $novoCodigoLancamento);
                        $this->insereRetiradaServico($row, $novoCodigoCardapio, $novoCodigoLancamento, $novoCodigoRetirada, $idutretiprat);
                        $produtos = $this->retornaProdutos($row, $servico['CDSERVFILI']);
    
                        if(!empty($produtos)) {
                            
                            foreach ($produtos as $produto) {
                                $this->insereItemEstoque($row, $produto, $novoCodigoLancamento, $seqItem);
                                $this->insereItemRetirada($row, $novoCodigoCardapio, $novoCodigoRetirada, $seqItem, $produto, $novoCodigoLancamento);
                                $seqItem = str_pad(($seqItem + 1), 4, "0", STR_PAD_LEFT); 
                            }
                        }
                        
                        $this->connection->commit();
                        $lancamentos[$novoCodigoLancamento] = $servico;
                        array_push($servicosComRetirada, $servico);
                        
                    } catch (\Exception $e){
                        $this->connection->rollback();
                        $response->setError(new Error($e->getMessage(), 1));
                    }
                } else {
                    array_push($servicosComRetirada, $servico);
                }
            }
            foreach($lancamentos as $nrlancestq => $servico){
                try{
                    $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($this->environment->getCdFilial(), $nrlancestq, $this->environment->getNrOrgTrab());
                    $this->validacoesEstoque->setLanctoestoq($lanctoestoq);
                    $this->validacoesEstoque->atualizaEstoque($this->environment->getCdFilial(), $nrlancestq, false, true, false);
                    $irregularProductCodes = $this->validacoesEstoque->getIrregularProducts();
                    if(!empty($irregularProductCodes)){
                        
                        $servico['IRREGULAR_PRODUCTS'] = $this->getProductsByCdProduto($irregularProductCodes);
                        $servicoFalho["SERVICO_FALHO"][] = $servico;
                    }
                } catch (\Exception $e){
                    $ERRO_ATUALIZA_ESTOQUE[] = $e->getMessage();
                    $servicoFalho["SERVICO_FALHO"][] = $servico;
                }
            }
            // $this->existeRetiradaAutomatica($row, $response);
            $retirada = array(
                "RETIRADA"       => !empty($servicosComRetirada),
                "SERVICES_ARRAY" => $servicosComRetirada
            );
            $response->addDataSet(new DataSet("dataRetirada", $retirada));
            
            if(!empty($ERRO_ATUALIZA_ESTOQUE)){
                $response->addDataSet(new DataSet("ERRO_ATUALIZA_ESTOQUE", $ERRO_ATUALIZA_ESTOQUE));
            }
            
            $response->addDataSet(new DataSet("servicoFalho", $servicoFalho));
            
        }else{
            throw new \Exception('Não há quantidade consumida cadastrada para nenhum produto!');
        }
    }
    
    private function getProductsByCdProduto($arrCdProduto){
        $products = array();
        $sql = "SELECT CDARVPROD, CDPRODUTO, NMPRODUTO FROM PRODUTO WHERE CDPRODUTO = :P_CDPRODUTO AND NRORG = :P_NRORG";
        foreach($arrCdProduto as $cdproduto){
            $stmt = $this->connection->prepare($sql);
            $stmt->execute(array(
                'P_CDPRODUTO' => $cdproduto,
                'P_NRORG' => $this->environment->getNrOrgTrab()
            ));
            $result = $stmt->fetch();
            if($result){
                $products[] = $result;
            }
        }
        return $products;
    }
    
    private function insereCardapio($row, $nrcardapio, $servico) {
        $query = "INSERT INTO CARDADIA (CDFILIAL , NRCARDAPIO, CDSERVFILI, CDOPERADOR, DTCARDAPIO, DTMESCARD) VALUES (:CDFILIAL , :NRCARDAPIO, :CDSERVFILI, :CDOPERADOR, :DTCARDAPIO, :DTMESCARD)";
        $stmt = $this->connection->prepare($query);
        $paramsInsertCardapio = array(
            "CDFILIAL"   => $row['CDFILIAL_FILTRADA'],
            "CDSERVFILI" => $servico['CDSERVFILI'],
            "CDOPERADOR" => $this->environment->getCdOperador(),
            "NRCARDAPIO" => $nrcardapio,
            "DTCARDAPIO" => $row['DTRETGERAL'],
            "DTMESCARD"  => $row['DTMESCARD']
        );
          //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL_FILTRADA'],'RETIRADA');
        $stmt->execute($paramsInsertCardapio);
    }
    
    private function insereLancamento($row, $servico, $nrlancestq) {
        $query = "INSERT INTO LANCTOESTOQ ( CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, CDFILIMOVI, CDOPERADOR ) VALUES ( :CDFILIAL, :NRLANCESTQ, :IDTPLANCTO, :DTLANCESTQ, :CDFILIMOVI, :CDOPERADOR )";
        $stmt = $this->connection->prepare($query);
        $paramsInsertCardapio = array(
            "CDFILIAL"     => $this->environment->getCdFilial(),
            "NRLANCESTQ"   => $nrlancestq,
            "IDTPLANCTO"   => "3",
            "DTLANCESTQ"   => $row['DTRETGERAL'],
            "CDFILIMOVI"   => $row['CDFILIAL_FILTRADA'],
            "CDOPERADOR"   => $this->environment->getCdOperador()
        );
          //coleta de dados
            $this->libUtilities->saveCollectedData($this->environment->getCdFilial(),'RETIRADA');
        $stmt->execute($paramsInsertCardapio);
    }

    private function insereRetiradaServico($row, $nrcardapio, $nrlancestq, $nrretirada, $idutretiprat) {

        $query = "INSERT INTO RETIRADA (CDFILIAL , NRCARDAPIO , NRRETIRADA , DTRETIRADA, NRLANCESTQ, NRREQUEXTRRE, CDFILANCESTQ, IDRETIPRAT) VALUES (:CDFILIAL , :NRCARDAPIO , :NRRETIRADA , :DTRETIRADA, :NRLANCESTQ, :NRREQUEXTRRE, :CDFILANCESTQ, :IDRETIPRAT)";
        $stmt = $this->connection->prepare($query);
        $paramsInsertCardapio = array(
            "CDFILIAL"     => $row['CDFILIAL_FILTRADA'],
            "NRCARDAPIO"   => $nrcardapio,
            "NRRETIRADA"   => $nrretirada,
            "DTRETIRADA"   => $row['DTRETGERAL'],
            "NRLANCESTQ"   => $nrlancestq,
            "NRREQUEXTRRE" => '0000000000',
            "CDFILANCESTQ" => $this->environment->getCdFilial(),
            "IDRETIPRAT"   => $idutretiprat
        );
         //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL_FILTRADA'],'RETIRADA');
        $stmt->execute($paramsInsertCardapio);
    }
    
    private function retornaProdutos($row, $servico) {
        
        $query = "SELECT NRRETGERAL, CDARVPROD, NMPRODUTO, SGUNIDADE, CDTIPORETI, NMTIPORETI, CDPRODUTO, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA, VRUNILANCTO, CDFILICPLA, DTCONSPLAN, NRCONSPLAN, CDSERVCPLA, NMSERVICO, CDTIPOPRAT, NMTIPOPRAT, CDPRATO, NMPRATO, CDFILAUXPRAT, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, VRSALDOEST FROM ( SELECT RE.NRRETGERAL, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, TR.CDTIPORETI, TR.NMTIPORETI, PR.CDPRODUTO, SUM(RE.QTPRERETIGER) AS QTPREVISRETI, RE.QTEXTRETIGER AS QTEXTRARETI, RE.QTDEVRETIGER AS QTDEVOLVRETI, CASE WHEN NVL(PR.VRFATOCONV,0) = 0 THEN RE.QTCONRETIGER ELSE ROUND(PR.VRFATOCONV * RE.QTCONRETIGER,5) END AS QTRETIRADA, RE.VRUNIRETGER AS VRUNILANCTO, RE.CDFILIAL AS CDFILICPLA, RE.DTRETGERAL AS DTCONSPLAN, '' AS NRCONSPLAN, RE.CDSERVFILI AS CDSERVCPLA, S.NMSERVICO, RE.CDTIPOPRAT, TP.NMTIPOPRAT, RE.CDPRATO, PA.NMPRATO, RE.CDFILAUXPRAT, RE.CDALMOXARIFE, RE.CDLOCALESTOQ, RE.NRLOTEESTQ, RE.NRSUBLOTE, DECODE(:IDSALDOEST,'S',QTDE_PRODUTO_LCEST(:CDFILIAL,PR.CDPRODUTO,:DTRETGERAL,'N',' ',' ',' '),0) AS VRSALDOEST FROM PRODUTO PR, TIPORETI TR, SERVICO S, RETIRADAGERAL RE, TIPOPRAT TP, PRATO PA WHERE ( RE.CDFILIAL =:CDFILIAL ) AND ( RE.DTRETGERAL =:DTRETGERAL ) AND ( PR.CDPRODUTO = RE.CDPRODUTO ) AND ( TR.CDTIPORETI = RE.CDTIPORETI ) AND ( S.CDFILIAL (+) = RE.CDFILIAL ) AND ( S.CDSERVFILI (+) = RE.CDSERVFILI ) AND ( TP.CDTIPOPRAT (+) = RE.CDTIPOPRAT ) AND ( PA.CDFILAUXPRAT (+) = RE.CDFILAUXPRAT ) AND ( PA.CDPRATO (+) = RE.CDPRATO ) AND ( RE.CDSERVFILI =:CDSERVFILI ) GROUP BY RE.NRRETGERAL, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, TR.CDTIPORETI, TR.NMTIPORETI, PR.CDPRODUTO, RE.QTEXTRETIGER, RE.QTDEVRETIGER, RE.QTCONRETIGER, RE.VRUNIRETGER, RE.CDFILIAL, RE.DTRETGERAL, RE.CDSERVFILI, S.NMSERVICO, RE.CDTIPOPRAT, TP.NMTIPOPRAT, RE.CDPRATO, PA.NMPRATO, RE.CDFILAUXPRAT, RE.CDALMOXARIFE, RE.CDLOCALESTOQ, RE.NRLOTEESTQ, RE.NRSUBLOTE, PR.VRFATOCONV ) WHERE QTRETIRADA <> 0 AND QTRETIRADA IS NOT NULL ORDER BY CDFILICPLA, CDARVPROD, NMPRODUTO, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL" => $row['CDFILIAL_FILTRADA'],
            "IDSALDOEST" => 'S',
            "CDSERVFILI" => $servico,
            "DTRETGERAL" => $row['DTRETGERAL']
        );
        $stmt->execute($params);
        $prodServ = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        return $prodServ;
    }
    
    private function insereItemEstoque($row, $produto, $nrlancestq, $seqItem) {
        $query = "INSERT INTO ITLANCTOEST (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, QTLANCTOEST, QTTOTLANCTO, CDPRODMOVI, CDFILIMOVI, DTLANCMOVI, CDTIPORETI, CDOPERADOR, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, VRUNILANCTO, VRTOTLANCTO, IDEVOLESTOQ, IDTIPOMOVI) VALUES (:CDFILIAL, :NRLANCESTQ, :NRSEQUITEM, :CDPRODUTO, :QTLANCTOEST, :QTTOTLANCTO, :CDPRODMOVI, :CDFILIMOVI, :DTLANCMOVI, :CDTIPORETI, :CDOPERADOR, :CDALMOXARIFE, :CDLOCALESTOQ, :NRLOTEESTQ, :NRSUBLOTE, :VRUNILANCTO, :VRTOTLANCTO, :IDEVOLESTOQ, :IDTIPOMOVI)";
        $valorLancto = (((float)$produto['QTRETIRADA']) * ((float)$produto['VRUNILANCTO']));
        $stmt = $this->connection->prepare($query);
        $paramsInsertCardapio = array(
            "CDFILIAL"      => $this->environment->getCdFilial(),
            "NRLANCESTQ"    => $nrlancestq,
            "NRSEQUITEM"    => $seqItem,
            "CDPRODUTO"     => $produto['CDPRODUTO'],
            "QTLANCTOEST"   => $produto['QTRETIRADA'],
            "QTTOTLANCTO"   => $produto['QTRETIRADA'],
            "CDPRODMOVI"    => $produto['CDPRODUTO'],
            "CDFILIMOVI"    => $row['CDFILIAL_FILTRADA'],
            "DTLANCMOVI"    => $row['DTRETGERAL'],
            "CDTIPORETI"    => $produto['CDTIPORETI'],
            "CDOPERADOR"    => $row['CDOPERADOR'],
            "CDALMOXARIFE"  => $produto['CDALMOXARIFE'],
            "CDLOCALESTOQ"  => $produto['CDLOCALESTOQ'],
            "NRLOTEESTQ"    => $produto['NRLOTEESTQ'],
            "NRSUBLOTE"     => $produto['NRSUBLOTE'],
            "VRUNILANCTO"   => $produto['VRUNILANCTO'],
            "VRTOTLANCTO"   => $valorLancto,
            "IDEVOLESTOQ"   => "N",
            "IDTIPOMOVI"    => "3"
        );
         //coleta de dados
            $this->libUtilities->saveCollectedData($this->environment->getCdFilial(),'RETIRADA');
        $stmt->execute($paramsInsertCardapio);
    }
    
    private function insereItemRetirada($row, $nrcardapio, $nrretirada, $seqItem, $produto, $nrlancestq) {
        $query = "INSERT INTO ITEMRETI (CDFILIAL, NRCARDAPIO, NRRETIRADA, NRSEQUITRETI, CDPRODUTO, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA , CDTIPORETI, NRLANCESTQ, CDFIITLANEST, NRSEQUITEM, CDTIPOPRAT, CDFILAUXPRAT, CDPRATO) VALUES (:CDFILIAL, :NRCARDAPIO, :NRRETIRADA, :NRSEQUITRETI, :CDPRODUTO, :QTPREVISRETI, :QTEXTRARETI, :QTDEVOLVRETI, :QTRETIRADA, :CDTIPORETI, :NRLANCESTQ, :CDFIITLANEST, :NRSEQUITEM, :CDTIPOPRAT, :CDFILAUXPRAT, :CDPRATO)";
        $stmt = $this->connection->prepare($query);
        $paramsInsertCardapio = array(
            "CDFILIAL"      => $row['CDFILIAL_FILTRADA'],
            "NRCARDAPIO"    => $nrcardapio,
            "NRRETIRADA"    => $nrretirada,
            "NRSEQUITRETI"  => $seqItem,
            "CDPRODUTO"     => $produto['CDPRODUTO'],
            "QTPREVISRETI"  => $produto['QTPREVISRETI'],
            "QTEXTRARETI"   => $produto['QTEXTRARETI'],
            "QTDEVOLVRETI"  => $produto['QTDEVOLVRETI'],
            "QTRETIRADA"    => $produto['QTRETIRADA'],
            "CDTIPORETI"    => $produto['CDTIPORETI'],
            "NRLANCESTQ"    => $nrlancestq,
            "CDFIITLANEST"  => $this->environment->getCdFilial(),
            "NRSEQUITEM"    => $seqItem,
            "CDTIPOPRAT"    => $produto['CDTIPOPRAT'],
            "CDFILAUXPRAT"  => $produto['CDFILAUXPRAT'],
            "CDPRATO"       => $produto['CDPRATO']
        );
         //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL_FILTRADA'],'RETIRADA');
        $stmt->execute($paramsInsertCardapio);
    }
    
    /* FIM DA ROTINA DE RETIRADA AUTOMATICA */
    
    /* ROTINA DE INCLUSAO DE UM SERVICO */
    public function validaNovoServico(Request $request, Response $response) {
        $servicos = $request->getDataSet()->getRows();
        $infoEstoq = array();
        $product = array();
        if(!empty($servicos)){
            $row = $servicos[0];
            $cdalmoxarife = isset($row['CDALMOXARIFE']) ? $row['CDALMOXARIFE'] : " ";
            $cdlocalestoq = isset($row['CDLOCALESTOQ']) ? $row['CDLOCALESTOQ'] : " ";
            $nrloteestq = isset($row['NRLOTEESTQ']) ? $row['NRLOTEESTQ'] : " ";
            $nrsublote = isset($row['NRSUBLOTE']) ? $row['NRSUBLOTE'] : " ";
            $cdfilial = isset($row['CDFILIAL_FILTRADA']) ? $row['CDFILIAL_FILTRADA'] : $row['CDFILIAL'];
            $params = array(
                "P_CDFILIAL"      => $cdfilial,
                "P_CDPRODUTO"     => $row['CDPRODUTO'],
                "P_CDALMOXARIFE"  => $cdalmoxarife,
                "P_CDLOCALESTOQ"  => $cdlocalestoq,
                "P_DTPOSICAO"     => $row['DTRETGERAL'],
                "P_NRLOTEESTQ"    => $nrloteestq,
                "P_NRSUBLOTE"     => $nrsublote
            );
            $product = $this->getProduct($params);
            $infoEstoq = $params;
        }
        $this->connection->beginTransaction();
        try {
            foreach ($servicos as $servico) {
                $servico['VRUNILANCTO'] = $product['VRCUSTPROD'];
                if($servico['IDUTRETIPRAT'] != 'S' && (!isset($servico['CTRLALMOXARIFADO']) || !$servico['CTRLALMOXARIFADO'])) {
                    $this->verificaServicoSemUtilizacaoPrato($servico, $response);
                }
                $this->insereRetiradaGeral($servico, $response);
            }
            $this->connection->commit();
            $response->addDataSet(new DataSet("infoEstoq", $infoEstoq));
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function verificaTipoPratoCardapio($row, $response) {
        $query = "SELECT DISTINCT C.CDTIPOPRAT, TP.NMTIPOPRAT FROM CNFCAMES C, TIPOPRAT TP WHERE (C.CDTIPOPRAT = TP.CDTIPOPRAT ) AND (C.CDFILIAL = :P_CDFILIAL ) AND (C.CDSERVFILI = :P_CDSERVFILI ) AND (C.DTREFECNFMES = :P_DTREFECNFMES) AND ((C.CDTIPOPRAT = :P_CDTIPOPRAT) OR (:P_TCDTIPOPRAT = 'T')) ORDER BY C.CDTIPOPRAT";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "P_CDFILIAL"     => $row['CDFILIAL'],
            "P_CDSERVFILI"   => $row['CDSERVFILI'],
            "P_DTREFECNFMES" => $row['DTMESCARD'],
            "P_CDTIPOPRAT"   => $row['CDTIPOPRAT'],
            "P_TCDTIPOPRAT"  => ''
        );
        $stmt->execute($params);
        $tipoPratoCardapio = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(empty($tipoPratoCardapio)) {
            throw new \Exception("Tipo de prato não cadastrado no cardápio do serviço.");
        }
    }
    
    public function verificaPratoCardapio($row, $response) {
        $query = "SELECT DISTINCT CO.CDFILAUXPRAT, CO.CDPRATO, PT.NMPRATO FROM COMPCARD CO, CARDADIA CA, PRATO PT WHERE (CA.CDFILIAL = :P_CDFILIAL ) AND (CA.CDSERVFILI = :P_CDSERVFILI) AND (CA.DTCARDAPIO = :P_DTCARDAPIO) AND (CO.CDFILIAL = CA.CDFILIAL ) AND (CO.NRCARDAPIO = CA.NRCARDAPIO) AND (CO.CDTIPOPRAT = :P_CDTIPOPRAT) AND (PT.CDPRATO = CO.CDPRATO ) AND ((PT.CDFILAUXPRAT = CA.CDFILIAL ) OR (PT.CDFILAUXPRAT = 'P')) AND ((CO.CDPRATO = :P_CDPRATO ) OR (:P_TCDPRATO = 'T')) ORDER BY CO.CDPRATO";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "P_CDFILIAL"   => $row['CDFILIAL'],
            "P_CDPRATO"    => '',
            "P_CDSERVFILI" => $row['CDSERVFILI'],
            "P_DTCARDAPIO" => $row['DTRETGERAL'],
            "P_TCDPRATO"   => 'T',
            "P_CDTIPOPRAT" => $row['CDTIPOPRAT']
        );
        $stmt->execute($params);
        $tipoPratoCardapio = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        if(empty($tipoPratoCardapio)) {
            throw new \Exception("Prato não cadastrado no cardápio do serviço.");
        }
    }
    
    public function insereRetiradaGeral($row, $response) {
        $sequencial = $this->geraSequencial($row);
    
        $query = "INSERT INTO RETIRADAGERAL (NRRETGERAL, CDFILIAL, CDSERVFILI, DTRETGERAL, CDPRODUTO, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT, CDTIPORETI, QTPRERETIGER, QTEXTRETIGER, QTDEVRETIGER, VRUNIRETGER, QTCONRETIGER, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE) VALUES (:NRRETGERAL, :CDFILIAL, :CDSERVFILI, :DTRETGERAL, :CDPRODUTO, :CDTIPOPRAT, :CDPRATO, :CDFILAUXPRAT, :CDTIPORETI, :QTPRERETIGER, :QTEXTRETIGER, :QTDEVRETIGER, :VRUNIRETGER, :QTCONRETIGER, :CDALMOXARIFE, :CDLOCALESTOQ, :NRLOTEESTQ, :NRSUBLOTE)";
        $stmt = $this->connection->prepare($query);
        $cdalmoxarife = $row['CDALMOXARIFE'] ? $row['CDALMOXARIFE'] : ' ';
        $cdlocalestoq = $row['CDLOCALESTOQ'] ? $row['CDLOCALESTOQ'] : ' ';
        $nrloteestq   = $row['NRLOTEESTQ']   ? $row['NRLOTEESTQ']   : ' ';
        $nrsublote    = $row['NRSUBLOTE']    ? $row['NRSUBLOTE']    : ' ';
        $paramsInsert = array(
            "NRRETGERAL"    => $sequencial,
            "CDFILIAL"      => $row['CDFILIAL_FILTRADA'],
            "CDSERVFILI"    => $row['CDSERVFILI'],
            "DTRETGERAL"    => $row['DTRETGERAL'],
            "CDPRODUTO"     => $row['CDPRODUTO'],
            "CDTIPOPRAT"    => $row['CDTIPOPRAT'],
            "CDPRATO"       => $row['CDPRATO'],
            "CDFILAUXPRAT"  => $row['CDFILAUXPRAT'],
            "CDTIPORETI"    => $row['CDTIPORETI'],
            "QTPRERETIGER"  => $row['QTPREVISRETI'],
            "QTEXTRETIGER"  => $row['QTEXTRARETI'],
            "QTDEVRETIGER"  => $row['QTDEVOLVRETI'],
            "QTCONRETIGER"  => $row['QTRETIRADA'],
            "VRUNIRETGER"   => $row['VRUNILANCTO'],
            "CDALMOXARIFE"  => $cdalmoxarife,
            "CDLOCALESTOQ"  => $cdlocalestoq,
            "NRLOTEESTQ"    => $nrloteestq,
            "NRSUBLOTE"     => $nrsublote
        );
         //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL_FILTRADA'],'RETIRADA');
        $stmt->execute($paramsInsert);
    }
    
    public function verificaServicoSemUtilizacaoPrato($row, $response) {
        $query = "SELECT CDPRODUTO, CDSERVFILI FROM RETIRADAGERAL WHERE CDPRODUTO =:CDPRODUTO AND CDSERVFILI =:CDSERVFILI AND DTRETGERAL =:DTRETGERAL AND CDFILIAL =:CDFILIAL";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDPRODUTO"  => $row['CDPRODUTO'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTRETGERAL" => $row['DTRETGERAL'],
            "CDFILIAL"   => $row['CDFILIAL_FILTRADA']
        );
        $stmt->execute($params);
        $servicos = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        if(!empty($servicos)) {
            throw new \Exception("Serviço já cadastrado.");
        }
    }
    /* FIM ROTINA DE INCLUSAO DE UM SERVICO */

    public function verificaDuplicataSemAlmox($row, $response){
        $query = "SELECT
                    R.NRRETGERAL
                FROM
                    RETIRADAGERAL R
                WHERE
                    R.CDPRODUTO        = :CDPRODUTO
                    AND R.DTRETGERAL   = :DTRETGERAL
                    AND R.CDFILIAL     = :CDFILIAL
                    AND (R.CDALMOXARIFE IS NULL OR R.CDALMOXARIFE = ' ')
                    AND (R.CDLOCALESTOQ IS NULL OR R.CDLOCALESTOQ = ' ')
                    AND (R.NRLOTEESTQ   IS NULL OR R.NRLOTEESTQ = ' ')
                    AND EXISTS (SELECT 1 FROM SERVICO S WHERE S.CDFILIAL = R.CDFILIAL AND S.CDSERVFILI = R.CDSERVFILI 
                                                             AND ((INSTR(:P_SERVSEL, S.CDSERVFILI) > 0) OR (:P_SERVSEL = 'T'))
                                                             AND ((INSTR(:P_GRUPSEL, S.CDGRUPSERV) > 0) OR (:P_GRUPSEL = 'T')) )";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDPRODUTO"    => $row['CDPRODUTO'],
            "DTRETGERAL"   => $row['DTRETGERAL'],
            "CDFILIAL"     => $row['CDFILIAL_FILTRADA'],
            "P_SERVSEL" => $row['SERVSEL'],
            "P_GRUPSEL" => $row['GRUPSEL']
        );
        $stmt->execute($params);
        $produtos = $stmt->fetchAll();
        if(!empty($produtos)){
            $msg = 'O produto já foi cadastrado para esta retirada!';
            $response->addDataSet(new DataSet("errorMsg", array($msg)));
            throw new \Exception($msg);
        }
    }
    
    /* ROTINA DE VALIDACAO DE UM NOVO PRODUTO*/
    public function validaNovoProduto(Request $request, Response $response) {
        $row = $request->getRow();
        $errorProducts = array();
        $errorMsg = "";
        $row['NEWPRODUCT'] = true;
        if(!isset($row['CTRLALMOXARIFADO']) || !$row['CTRLALMOXARIFADO']){
            $this->verificaDuplicataSemAlmox($row, $response);
            $this->verificaEstoqueNegativo($row, $response, $errorProducts, $errorMsg);
            if(!empty($errorMsg)) {
                $response->addDataSet(new DataSet("errorMsg", array($errorMsg)));
                throw new \Exception($errorMsg);
            }
            if(!empty($errorProducts)) {
                $response->addDataSet(new DataSet("data", $errorProducts));
            }
        }else{
            try{
                $this->checkDuplicateRecord($row, $response);
                $infoProduct = array();
                array_push($infoProduct, array( "CDPRODUTO" => $row['CDPRODUTO'], "CDARVPROD" => $row['CDARVPROD'], "QTESTOQDIA" => $row['QTESTOQDIA'], "NMPRODUTO" => $row['NMPRODUTO']));
                $response->addDataSet(new DataSet("data", $infoProduct));
            }catch(\Exception $e){
                $response->setError(new Error($e->getMessage(), 500));
            }
        }
        $this->connection->beginTransaction();
        try {
            $controla = $this->verificaControleEstoque($row, $response);
            
            if($controla == 'N') {
                throw new \Exception("Inclusão bloqueada. Produto não é um produto com controle de estoque.");
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function verificaControleEstoque($row, $response) {
        $query = "SELECT IDCNTRESTOQ FROM PRODFILI WHERE CDFILIAL = :CDFILIAL AND CDPRODUTO = :CDPRODUTO";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL" => $row['CDFILIAL_FILTRADA'],
            "CDPRODUTO" => $row['CDPRODUTO']
        );
        $stmt->execute($params);
        $controla = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        if($controla) {
            return $controla[0]['IDCNTRESTOQ'];
        }
    }
    /* FIM DA ROTINA DE VALIDACAO DE UM NOVO PRODUTO*/

    
    /* ROTINA DE VALIDACAO DE CALCULO DE RETIRADA*/
    public function validaCalculoRetirada(Request $request, Response $response) {
        $row = $request->getRow();
        $this->validaDataRetirada($row, $response);
        
    }
    
    public function validaDataRetirada($row, $response) {
        try {
            $query = "SELECT NVL((SUM(QTCONRETIGER) + SUM(QTDEVRETIGER)), 0) AS QNTMOVI FROM RETIRADAGERAL WHERE ( CDFILIAL = :CDFILIAL ) AND ( DTRETGERAL = :DTRETGERAL ) AND ( ( INSTR(:P_SERVFILI, CDSERVFILI) > 0 ) OR ( :P_SERVFILI = 'T' ) ) AND EXISTS ( SELECT 1 FROM SERVICO S WHERE S.CDFILIAL = CDFILIAL AND S.CDSERVFILI = CDSERVFILI AND ( ( INSTR(:P_GRUPSERV, S.CDGRUPSERV) > 0 ) OR ( :P_GRUPSERV = 'T' ) ) )";
            $stmt = $this->connection->prepare($query);
            $params = array(
                "CDFILIAL" => $row['CDFILIAL'],
                "DTRETGERAL" => $row['DTRETGERAL'],
                "P_SERVFILI" => $row['NMSERVICO'],
                "P_GRUPSERV" => $row['NMGRUPSERV']
            );
            $stmt->execute($params);
            $dataRetirada = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            $dataRetirada = $dataRetirada[0]['QNTMOVI'];
            if($dataRetirada > 0) {
                $retirada = array(
                    "RETIRADA" => true
                );
                return $response->addDataSet(new DataSet("dataRetirada", $retirada));
            } else {
                $retirada = array(
                    "RETIRADA" => false
                );
                return $response->addDataSet(new DataSet("dataRetirada", $retirada));
            }
        }catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    /* FIM DA ROTINA DE VALIDACAO DE CALCULO DE RETIRADA*/


    /* ROTINA  DE CALCULO DE RETIRADA*/
    public function calculaRetirada(Request $request, Response $response) {
        $row = $request->getRow();
        $this->connection->beginTransaction();
        try {
            if ($row['CDSERVFILI'] == 'T') {
                $listaServicos = $this->validaServicos($row, $response);
            } else {
                $listaServicos = $row['FILISERV'];
            }
            if(!$listaServicos || (is_array($listaServicos) && empty($listaServicos))){
                throw new \Exception("NENHUM_SERVICO_CADASTRADO");
            }
            $nrlista = $this->util->insertLista($listaServicos);
            $isValidServices = $this->validaRetiradaServicos($row, $response, $nrlista);
            if($isValidServices){
                $this->validaRetiradaDefault($row, $response, $nrlista);
                $this->validaFalhaParam($row, $response);
                
                // $consplan = $this->executaProcedureConsumoPlanejado($row, $response, $request, $nrlista);
                $consplan   = $this->geraNovoCodigo('CONSPLAN'.$row['CDFILIAL']);
                $nrconsplan = $this->validaNewCodeConsplan($consplan);
                $this->executaProcedureRequisicaoAnalitica($row, $nrconsplan, $nrlista);
                $this->deletaRetiradaGeral($row, $response, $nrlista);
                $this->insereRetiradaGeralCalculada($row, $response, $nrconsplan);
                $this->util->deleteLista($nrlista);
                
                $this->connection->commit();
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function validaServicos($row, $response) {
        $query = "SELECT CDFILIAL || CDSERVFILI AS LISTASERVICO FROM SERVICO WHERE CDFILIAL = :CDFILIAL AND IDTIPOSERV <> 'A' AND IDSERVATIV <> 'N' AND ( ( IDTPSERVHOSP IS NULL ) OR ( IDTPSERVHOSP = '0' ) ) AND ( ( INSTR(:CDGRUPSERV, CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) AND ( ( INSTR(:P_CDGRUPSERVFILTRO, CDGRUPSERV) > 0 ) OR ( :P_CDGRUPSERVFILTRO = 'T' ) ) AND ( INSTR(:P_CDSERVFILI, CDSERVFILI) > 0 OR ( :P_TCDSERVFILI = 'T' ) ) AND NOT EXISTS ( SELECT SERVOPER.CDFILIAL FROM SERVOPER WHERE SERVOPER.CDFILIAL = :CDFILIAL AND SERVOPER.CDOPERADOR = :CDOPERADOR AND SERVOPER.CDFILIAL = SERVICO.CDFILIAL ) UNION ALL SELECT SERVOPER.CDFILIAL || SERVOPER.CDSERVFILI AS LISTASERVICO FROM SERVOPER, SERVICO WHERE SERVOPER.CDFILIAL = :CDFILIAL AND SERVOPER.CDOPERADOR = :CDOPERADOR AND SERVICO.CDFILIAL = SERVOPER.CDFILIAL AND SERVICO.CDSERVFILI = SERVOPER.CDSERVFILI AND SERVICO.IDTIPOSERV <> 'A' AND SERVICO.IDSERVATIV <> 'N' AND ( ( IDTPSERVHOSP IS NULL ) OR ( IDTPSERVHOSP = '0' ) ) AND ( ( INSTR(:CDGRUPSERV, SERVICO.CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) AND ( ( INSTR(:P_CDGRUPSERVFILTRO, SERVICO.CDGRUPSERV) > 0 ) OR ( :P_CDGRUPSERVFILTRO = 'T' ) ) AND ( INSTR(:P_CDSERVFILI, SERVICO.CDSERVFILI) > 0 OR ( :P_TCDSERVFILI = 'T' ) )";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL" => $row['CDFILIAL_FILTRADA'],
            "CDGRUPSERV" => $row['CDGRUPSERV'],
            "CDOPERADOR" => $this->environment->getCdOperador(),
            "P_TCDSERVFILI" => $row['TCDSERVFILI'],
            "P_CDSERVFILI" => $row['NMSERVICO'],
            "P_CDGRUPSERVFILTRO" => $row['NMGRUPSERV']
        );
        $stmt->execute($params);
        $servicosConcat = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        return array_column($servicosConcat, 'LISTASERVICO');
    }
    
    public function validaRetiradaServicos($row, $response, $lista) {
        $query = "SELECT DISTINCT CR.CDSERVFILI, SR.NMSERVICO FROM ITEMRETI IR, RETIRADA RT, CARDADIA CR, SERVICO SR, LISTA WHERE LISTA.NRLISTA = :NRLISTA AND SUBSTR(LISTA.CDLISTA, 1, 4) = CR.CDFILIAL AND SUBSTR(LISTA.CDLISTA, 5, 5) = CR.CDSERVFILI AND (IR.CDFILIAL = RT.CDFILIAL ) AND (IR.NRCARDAPIO = RT.NRCARDAPIO) AND (IR.NRRETIRADA = RT.NRRETIRADA) AND (IR.QTRETIRADA > 0 ) AND (CR.DTCARDAPIO = :DTRETIRADA) AND (CR.CDFILIAL = RT.CDFILIAL ) AND (CR.NRCARDAPIO = RT.NRCARDAPIO) AND (CR.CDFILIAL = SR.CDFILIAL ) AND (CR.CDSERVFILI = SR.CDSERVFILI)";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "NRLISTA" => $lista,
            "DTRETIRADA" => $row['DTRETGERAL']
        );
        $stmt->execute($params);
        $servicoRetirada = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(!empty($servicoRetirada)) {
            $response->addDataSet(new DataSet("services", $servicoRetirada));
            return false;
        }
        return true;
    }
    
    public function validaRetiradaDefault($row, $response, $lista) {
        $query = "SELECT DISTINCT IDUTREQUPRAT, CDTPRCONSFIX, CDPRACONSFIX, CDTPRCONSPER, CDPRACONSPER, CDFILIAL, CDPARPLANEJA, IDORDEREQU, NRNIVEREQU, CDTPRETIPADR, IDVISUAQTEST FROM PARAMPLANEJA, LISTA WHERE LISTA.NRLISTA = :NRLISTA AND PARAMPLANEJA.CDFILIAL = SUBSTR(LISTA.CDLISTA, 1, 4)";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "NRLISTA" => $lista
        );
        $stmt->execute($params);
        $default = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        if($default[0]['CDTPRETIPADR'] == NULL) {
            throw new \Exception("Operação bloqueada! Não há Tipo de Retirada Padrão cadastrada."); // exibir os servicos concatenados
        }
    }
    
    public function validaFalhaParam($row, $response) {
        $query = "SELECT DISTINCT IDUTREQUPRAT, CDTPRCONSFIX, CDPRACONSFIX, CDTPRCONSPER, CDPRACONSPER, CDFILIAL, CDPARPLANEJA, IDORDEREQU, NRNIVEREQU, CDTPRETIPADR, IDVISUAQTEST FROM PARAMPLANEJA WHERE CDFILIAL = :CDFILIAL";
        $stmt = $this->connection->prepare($query);
        $paramsFalha = array(
            "CDFILIAL" => $row['CDFILIAL']
        );
        $stmt->execute($paramsFalha);
        $falhaVerificacao = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        if(!$falhaVerificacao[0]['CDTPRCONSFIX']) {
            throw new \Exception("Tipo de prato default de consumo fixo deve ser parametrizado");
        }
        if(!$falhaVerificacao[0]['CDPRACONSFIX']) {
            throw new \Exception("Prato default de consumo fixo deve ser parametrizado");
        }
        if(!$falhaVerificacao[0]['CDTPRCONSPER']) {
            throw new \Exception("Tipo de prato default de consumo percapita deve ser parametrizado");
        }
        if(!$falhaVerificacao[0]['CDPRACONSPER']) {
            throw new \Exception("Prato default de consumo percapita deve ser parametrizado");
        }
    }
    
    public function deletaRetiradaGeral($row, $response, $nrlista) {
        $retiradas = $this->buscaRetiradaGeral($row, $response, $nrlista);
    
        if(!empty($retiradas)) {
            $query = "DELETE FROM RETIRADAGERAL R WHERE R.CDFILIAL = :CDFILIAL AND R.DTRETGERAL = :DTRETGERAL AND EXISTS (SELECT 1 FROM LISTA L WHERE L.NRLISTA = :NRLISTA AND L.CDLISTA = R.CDFILIAL||R.CDSERVFILI)";
            $stmt = $this->connection->prepare($query);
            $params = array(
                "CDFILIAL"   => $row['CDFILIAL'],
                "DTRETGERAL" => $row['DTRETGERAL'],
                "NRLISTA" => $nrlista
            );
            $stmt->execute($params);
    	}
    }
    
    public function buscaRetiradaGeral($row, $response, $nrlista) {
        $query = "SELECT CDFILIAL, DTRETGERAL FROM RETIRADAGERAL R WHERE R.CDFILIAL = :CDFILIAL AND R.DTRETGERAL = :DTRETGERAL AND EXISTS (SELECT 1 FROM LISTA L WHERE L.NRLISTA = :NRLISTA AND L.CDLISTA = R.CDFILIAL||R.CDSERVFILI)";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "DTRETGERAL" => $row['DTRETGERAL'],
            "NRLISTA" => $nrlista
        );
        $stmt->execute($params);
        $retiradas = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        return $retiradas;
    }
    
    public function insereRetiradaGeralCalculada($row, $response, $consplan) {
        $sequencial = $this->geraSequencial($row);
        $query = "INSERT INTO RETIRADAGERAL( NRRETGERAL, CDFILIAL, CDSERVFILI, DTRETGERAL, CDPRODUTO, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT, CDTIPORETI, QTPRERETIGER, QTEXTRETIGER, QTDEVRETIGER, VRUNIRETGER, QTCONRETIGER, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE ) ( SELECT LPAD(ROWNUM, 10, 0) AS NRRETGERAL, A.CDFILIAL, A.CDSERVFILI, A.DTRETGERAL, A.CDPRODUTO, A.CDTIPOPRAT, A.CDPRATO, A.CDFILAUXPRAT, A.CDTIPORETI, A.QTPRERETIGER, 0.0 VRUNIRETGER, A.QTEXTRETIGER, A.QTDEVRETIGER, A.QTCONRETIGER, A.CDALMOXARIFE, A.CDLOCALESTOQ, A.NRLOTEESTQ, A.NRSUBLOTE FROM ( SELECT CP.CDFILICPLA AS CDFILIAL, CP.CDSERVCPLA AS CDSERVFILI, CP.DTCONSPLAN AS DTRETGERAL, CP.CDPRODCPLA AS CDPRODUTO, DECODE( SE.IDUTRETIPRAT, 'S', CP.CDTPPRATCPLA, NULL ) AS CDTIPOPRAT, DECODE( SE.IDUTRETIPRAT, 'S', CP.CDPRATCONSPL, NULL ) AS CDPRATO, DECODE( SE.IDUTRETIPRAT, 'S', CP.CDFAPRATCPLA, NULL ) AS CDFILAUXPRAT, TR.CDTIPORETI, SUM(CP.QTCONSPLAN) AS QTPRERETIGER, 0.0 AS QTEXTRETIGER, 0.0 AS QTDEVRETIGER, 0.0 AS QTCONRETIGER, NVL(PF.CDALMOXARIFE, ' ') AS CDALMOXARIFE, NVL(PF.CDLOCALESTOQ, ' ') AS CDLOCALESTOQ, ' ' AS NRLOTEESTQ, ' ' AS NRSUBLOTE FROM CONSPLAN CP, TIPORETI TR, PARAMPLANEJA PJ, SERVICO SE, PARAMFILIAL PF WHERE (CP.CDFILICPLA = :CDFILICPLA) AND (CP.NRCONSPLAN = :NRCONSPLAN) AND (CP.DTCONSPLAN = :DTCONSPLAN) AND (SE.CDFILIAL = CP.CDFILICPLA) AND (SE.CDSERVFILI = CP.CDSERVCPLA) AND ( CP.IDCONSPLAN IN ( SUBSTR(:IDCONSUMO, 1, 1), SUBSTR(:IDCONSUMO, 2, 1), SUBSTR(:IDCONSUMO, 3, 1) ) ) AND (TR.CDTIPORETI = PJ.CDTPRETIPADR) AND (PJ.CDFILIAL = CP.CDFILICPLA) AND (PF.CDFILIAL = CP.CDFILICPLA) GROUP BY CP.CDFILICPLA, CP.CDSERVCPLA, CP.DTCONSPLAN, CP.CDPRODCPLA, DECODE( SE.IDUTRETIPRAT, 'S', CP.CDTPPRATCPLA, NULL ), DECODE( SE.IDUTRETIPRAT, 'S', CP.CDPRATCONSPL, NULL ), DECODE( SE.IDUTRETIPRAT, 'S', CP.CDFAPRATCPLA, NULL ), TR.CDTIPORETI, PF.CDALMOXARIFE, PF.CDLOCALESTOQ ) A )";
        $stmt = $this->connection->prepare($query);
        $paramsInsertCalculado = array(
            "CDFILICPLA"      => $row['CDFILIAL_FILTRADA'],
            "NRCONSPLAN"    => $consplan,
            "DTCONSPLAN"    => $row['DTRETGERAL'],
            "IDCONSUMO"     => $row['CONSUMO']
        );
         //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL_FILTRADA'],'RETIRADA');
        $stmt->execute($paramsInsertCalculado);
    }
    
    public function geraSequencial($row) {
        $query = "SELECT MAX(NRRETGERAL) AS NRRETGERAL FROM RETIRADAGERAL WHERE (CDFILIAL = :P_CDFILIAL ) AND (DTRETGERAL = :P_DTRETGERAL)";
        $stmt = $this->connection->prepare($query);
        $arrayParams = array(
            "P_CDFILIAL" => $row['CDFILIAL_FILTRADA'],
            "P_DTRETGERAL" => $row['DTRETGERAL']
        );
        $stmt->execute($arrayParams);
        $retirada = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        $sequencial = $this->validaSequencial($retirada[0]['NRRETGERAL']);
        return $sequencial;
    }
    
    public function validaSequencial($sequencial) {
        $sequencial = str_pad(($sequencial + 1), 10, "0", STR_PAD_LEFT); 
        return $sequencial;
    }
    
    public function validaNewCodeConsplan($sequencialConsplan) {
        $seqConsplan = substr($sequencialConsplan, 1); 
        $seqConsplan = 'R'.$seqConsplan;
        return $seqConsplan;
    }
    /* FIM DA ROTINA  DE CALCULO DE RETIRADA*/
    
    
    /* ROTINA  ALTERACAO DE SERVICOS*/
    public function alteraServicos(Request $request, Response $response) {
        $servs = $request->getDataSet()->getRows();
        $this->connection->beginTransaction();
        try {
            $errorProducts = array();
            $errorMsg = "";
            foreach ($servs as $serv) {
                if(isset($serv['changed']) && $serv['changed'] === true){
                    $verificaEstoque = true;
                    if(!isset($serv['CTRLALMOXARIFADO']) || !$serv['CTRLALMOXARIFADO']){
                        $verificaEstoque = $this->verificaEstoqueNegativo($serv, $response, $errorProducts, $errorMsg);
                    }
                    $fator = 1;
                    $novasQuantidades = $this->calculaCamposQuantidade($serv, $serv, $fator, $response);
                    $serv['QTRETIRADA'] = round(($novasQuantidades['QTRETIRADA']),5);
                    $serv['QTEXTRARETI'] = round(($novasQuantidades['QTEXTRARETI']),5);
                    $serv['QTDEVOLVRETI'] = round(($novasQuantidades['QTDEVOLVRETI']), 5);
                    $serv['CDALMOXARIFE_BACKUP'] = $serv['CDALMOXARIFE'];
                    $serv['CDLOCALESTOQ_BACKUP'] = $serv['CDLOCALESTOQ'];
                    $serv['NRLOTEESTQ_BACKUP'] = $serv['NRLOTEESTQ'];
                    $serv['NRSUBLOTE_BACKUP'] = $serv['NRSUBLOTE'];
                    $this->updataRetiradaGeral($serv, $serv, $response, $serv['DTRETGERAL']);
                }
            }
            if($errorMsg)
                throw new \Exception("Quantidade informada não disponível no estoque.<br>");
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    /* ROTINA  ALTERACAO DE SERVICOS*/
    
    /* ROTINA  ALTERACAO DE PRODUTOS*/
    public function alteraProdutos(Request $request, Response $response) {
        $dif  = 0;
        $errorProducts = array();
        $errorMsg = "Quantidade informada não disponível no estoque.<br>";
        $produtos = $request->getDataSet()->getRows();
        $this->connection->beginTransaction();
        try {
            foreach ($produtos as $produto) {
                $soma = 0;
                $produto['UPDATEPRODUCT'] = true;
                if(isset($produto['ALTERAALMOXLOCLOTE']) && $produto['ALTERAALMOXLOCLOTE'] === true){
                    $this->checkDuplicateRecord($produto, $response);
                }
                $ultimaQtRetiradaProduto = $this->qtRetiradaPorProduto($response, $produto);
                if($this->verificaEstoqueNegativo($produto, $response, $errorProducts, $errorMsg) || $produto['CTRLALMOXARIFADO']) {
                    if(isset($produto['changed']) && $produto['changed'] === true){
                        $servicos = $this->retiradaProduto($produto, $response);
                        $length = count($servicos);
                        foreach ($servicos as $index => $servico) {
                            if($ultimaQtRetiradaProduto == 0) {
                                if($produto['QTPREVISRETI'] == 0){
                                    $fator = 1/$length;
                                } else {
                                    $fator = $servico['QTPREVISRETI']/$produto['QTPREVISRETI'];
                                }
                            } else {
                                $fator = $servico['QTRETIRADA'] / $ultimaQtRetiradaProduto;
                            }
                            $novasQuantidades = $this->calculaCamposQuantidade($servico, $produto, $fator, $response);
                            $servico['QTRETIRADA'] = round(($novasQuantidades['QTRETIRADA']),5);
                            $servico['QTEXTRARETI'] = round(($novasQuantidades['QTEXTRARETI']),5);
                            $servico['QTDEVOLVRETI'] = round(($novasQuantidades['QTDEVOLVRETI']), 5);
                            $soma += $servico['QTRETIRADA'];
                            if($soma > 0 && $soma != $produto['QTRETIRADA'] && ($index === $length-1)){
                                $dif = $produto['QTRETIRADA'] - $soma;
                                $servico['QTRETIRADA'] = round(($novasQuantidades['QTRETIRADA'] + $dif),5);
                                $servico['QTEXTRARETI'] = round(($novasQuantidades['QTEXTRARETI'] + $dif),5);
                            }
                            $this->updataRetiradaGeral($servico, $produto, $response, $produto['DTRETGERAL']);
                        }
                    }
                }
            }
            $this->connection->commit();
            if(!empty($errorProducts)) {
                $response->addDataSet(new DataSet("produtosInvalidos", $errorProducts));
                $response->addDataSet(new DataSet("errorMsg", array($errorMsg)));
                throw new \Exception($errorMsg);
            }
        } catch (\Exception $e) {
            $this->realizaRollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function retiradaProduto($row, $response) {
        $query = "SELECT NRRETGERAL, CDARVPROD, NMPRODUTO, SGUNIDADE, CDTIPORETI, NMTIPORETI, CDPRODUTO, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA, VRUNILANCTO, CDFILICPLA, DTCONSPLAN, NRCONSPLAN, CDSERVCPLA, NMSERVICO, CDTIPOPRAT, NMTIPOPRAT, CDPRATO, NMPRATO, CDFILAUXPRAT, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, VRSALDOEST FROM ( SELECT RE.NRRETGERAL, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, TR.CDTIPORETI, TR.NMTIPORETI, PR.CDPRODUTO, SUM(RE.QTPRERETIGER) AS QTPREVISRETI, RE.QTEXTRETIGER      AS QTEXTRARETI, RE.QTDEVRETIGER      AS QTDEVOLVRETI, RE.QTCONRETIGER      AS QTRETIRADA, RE.VRUNIRETGER       AS VRUNILANCTO, RE.CDFILIAL          AS CDFILICPLA, RE.DTRETGERAL        AS DTCONSPLAN, ''                   AS NRCONSPLAN, RE.CDSERVFILI        AS CDSERVCPLA, S.NMSERVICO, RE.CDTIPOPRAT, TP.NMTIPOPRAT, RE.CDPRATO, PA.NMPRATO, RE.CDFILAUXPRAT, RE.CDALMOXARIFE, RE.CDLOCALESTOQ, RE.NRLOTEESTQ, RE.NRSUBLOTE, DECODE(:IDSALDOEST, 'S', QTDE_PRODUTO_LCEST(:CDFILIAL, PR.CDPRODUTO, :DTRETGERAL, 'N', ' ', ' ', ' '), 0) AS VRSALDOEST FROM PRODUTO       PR, TIPORETI      TR, SERVICO       S, RETIRADAGERAL RE, TIPOPRAT      TP, PRATO         PA WHERE ( RE.CDFILIAL = :CDFILIAL ) AND ( RE.DTRETGERAL = :DTRETGERAL ) AND ( PR.CDPRODUTO = RE.CDPRODUTO ) AND ( TR.CDTIPORETI = RE.CDTIPORETI ) AND ( S.CDFILIAL (+) = RE.CDFILIAL ) AND ( S.CDSERVFILI (+) = RE.CDSERVFILI ) AND ( TP.CDTIPOPRAT (+) = RE.CDTIPOPRAT ) AND ( PA.CDFILAUXPRAT (+) = RE.CDFILAUXPRAT ) AND ( PA.CDPRATO (+) = RE.CDPRATO ) AND ( RE.CDPRODUTO = :CDPRODUTO ) AND ( ( INSTR(:P_SERVSEL, RE.CDSERVFILI) > 0 ) OR ( :P_SERVSEL = 'T' ) ) AND ( ( INSTR(:P_GRUPSEL, S.CDGRUPSERV) > 0 ) OR ( :P_GRUPSEL = 'T' ) ) AND (NVL(RE.CDALMOXARIFE, ' ') = :CDALMOXARIFE_BACKUP) AND (NVL(RE.CDLOCALESTOQ, ' ') = :CDLOCALESTOQ_BACKUP) AND (NVL(RE.NRLOTEESTQ, ' ') = :NRLOTEESTQ_BACKUP) AND (NVL(RE.NRSUBLOTE, ' ') = :NRSUBLOTE_BACKUP) GROUP BY RE.NRRETGERAL, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, TR.CDTIPORETI, TR.NMTIPORETI, PR.CDPRODUTO, RE.QTEXTRETIGER, RE.QTDEVRETIGER, RE.QTCONRETIGER, RE.VRUNIRETGER, RE.CDFILIAL, RE.DTRETGERAL, RE.CDSERVFILI, S.NMSERVICO, RE.CDTIPOPRAT, TP.NMTIPOPRAT, RE.CDPRATO, PA.NMPRATO, RE.CDFILAUXPRAT, RE.CDALMOXARIFE, RE.CDLOCALESTOQ, RE.NRLOTEESTQ, RE.NRSUBLOTE ) ORDER BY CDFILICPLA, CDARVPROD, NMPRODUTO, CDTIPOPRAT, CDPRATO, CDFILAUXPRAT";
        $CDALMOXARIFE_BACKUP = isset($row['CDALMOXARIFE_BACKUP']) ? $row['CDALMOXARIFE_BACKUP'] : ' ';
        $CDLOCALESTOQ_BACKUP = isset($row['CDLOCALESTOQ_BACKUP']) ? $row['CDLOCALESTOQ_BACKUP'] : ' ';
        $NRLOTEESTQ_BACKUP = isset($row['NRLOTEESTQ_BACKUP']) ? $row['NRLOTEESTQ_BACKUP'] : ' ';
        $NRSUBLOTE_BACKUP = isset($row['NRSUBLOTE_BACKUP']) ? $row['NRSUBLOTE_BACKUP'] : ' ';
        $params = array(
            "IDSALDOEST" => 'S',
            "CDFILIAL" => $row['CDFILIAL_FILTRADA'],
            "CDPRODUTO" => $row['CDPRODUTO'],
            "DTRETGERAL" => $row['DTRETGERAL'],
            "P_SERVSEL" => $row['SERVSEL'],
            "P_GRUPSEL" => $row['GRUPSEL'],
            "CDALMOXARIFE_BACKUP" => $CDALMOXARIFE_BACKUP,
            "CDLOCALESTOQ_BACKUP" => $CDLOCALESTOQ_BACKUP,
            "NRLOTEESTQ_BACKUP" => $NRLOTEESTQ_BACKUP,
            "NRSUBLOTE_BACKUP" => $NRSUBLOTE_BACKUP
        );
        $servicos = $this->connection->fetchAll($query, $params);
        return $servicos;
    }
    
    public function qtRetiradaPorProduto($response, $produto) {
        $query = "SELECT NVL(SUM(R.QTCONRETIGER), 0) AS QTCONRETIGER
                    FROM RETIRADAGERAL R
                    WHERE R.CDPRODUTO =:CDPRODUTO
                       AND TRUNC(R.DTRETGERAL) = TO_DATE(:DTRETGERAL,'DD/MM/YYYY')
                       AND (R.CDALMOXARIFE = :CDALMOXARIFE OR (:CDALMOXARIFE = ' ' AND R.CDALMOXARIFE IS NULL))
                       AND (R.CDLOCALESTOQ = :CDLOCALESTOQ OR (:CDLOCALESTOQ = ' ' AND R.CDLOCALESTOQ IS NULL))
                       AND (R.NRLOTEESTQ = :NRLOTEESTQ OR (:NRLOTEESTQ = ' ' AND R.NRLOTEESTQ IS NULL))
                       AND (R.NRSUBLOTE = :NRSUBLOTE OR (:NRSUBLOTE = ' ' AND R.NRSUBLOTE IS NULL))
                       AND R.CDFILIAL = :CDFILIAL
                       AND EXISTS (SELECT 1 FROM SERVICO S WHERE S.CDFILIAL = R.CDFILIAL AND S.CDSERVFILI = R.CDSERVFILI 
                                                             AND ((INSTR(:P_SERVSEL, S.CDSERVFILI) > 0) OR (:P_SERVSEL = 'T'))
                                                             AND ((INSTR(:P_GRUPSEL, S.CDGRUPSERV) > 0) OR (:P_GRUPSEL = 'T')) )";
        $params = array(
            "CDPRODUTO" => $produto['CDPRODUTO'],
            "DTRETGERAL" => $produto['DTRETGERAL'],
            "CDALMOXARIFE" => $produto['CDALMOXARIFE'],
            "CDLOCALESTOQ" => $produto['CDLOCALESTOQ'],
            "NRLOTEESTQ"   => $produto['NRLOTEESTQ'],
            "NRSUBLOTE"   => $produto['NRSUBLOTE'],
            "CDFILIAL"    => $produto['CDFILIAL_FILTRADA'],
            "P_SERVSEL" => $produto["SERVSEL"],
            "P_GRUPSEL" => $produto["GRUPSEL"]
        );
        $ultimaQtRetiradaProduto = $this->connection->fetchAll($query, $params)[0]['QTCONRETIGER'];
        return $ultimaQtRetiradaProduto; 
    }
    
    public function calculaCamposQuantidade($servico, $produto, $fator, $response) {
        $consumida = ($produto['QTRETIRADA'] * $fator);
        $extra = 0.0;
        $devolvida = 0.0;
        $previstaServico = $servico['QTPREVISRETI'];
        if($previstaServico == 0){
            $extra = $consumida;
        } else if($previstaServico < $consumida){
            $extra = ($consumida - $previstaServico);    
        }
        if($consumida < $previstaServico){
            $devolvida = $previstaServico - $consumida;
        }
        
        $quantidades = array(
            "QTEXTRARETI"  => $extra,
            "QTDEVOLVRETI" => $devolvida,
            "QTRETIRADA"   => $consumida
        );
        return $quantidades;
    }
    /* FIM DA ROTINA  ALTERACAO DE PRODUTOS*/
    
    /* ROTINA  ALTERACAO DE TIPO DE RETIRADA*/
    public function alteraTipoRetirada(Request $request, Response $response) {
        $produtos = $request->getDataSet()->getRows();
        foreach ($produtos as $produto) {
            $this->updataTipoRetirada($produto, $response);
        }
    }
    
    public function updataTipoRetirada($row, $response) {
        try {
            $query = "UPDATE RETIRADAGERAL SET CDTIPORETI =:CDTIPORETI WHERE CDFILIAL =:CDFILIAL AND DTRETGERAL = :DTRETGERAL AND CDPRODUTO =:CDPRODUTO";
            $stmt = $this->connection->prepare($query);
            $params = array(
                "CDTIPORETI"    => $row['CDTIPORETI'],
                "CDFILIAL"      => $row['CDFILIAL'],
                "DTRETGERAL"    => $row['DTRETGERAL'],
                "CDPRODUTO"     => $row['CDPRODUTO']
            );
             //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
            $stmt->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    /* FIM DA ROTINA  ALTERACAO DE TIPO DE RETIRADA*/
    
    private function setParamsEstoq(&$row){
        $row['CDALMOXARIFE'] = isset($row['CDALMOXARIFE']) && $row['CDALMOXARIFE'] ? $row['CDALMOXARIFE'] : ' '; 
        $row['CDLOCALESTOQ'] = isset($row['CDLOCALESTOQ']) && $row['CDLOCALESTOQ'] ? $row['CDLOCALESTOQ'] : ' '; 
        $row['NRLOTEESTQ']   = isset($row['NRLOTEESTQ'])   && $row['NRLOTEESTQ']   ? $row['NRLOTEESTQ']   : ' '; 
        $row['NRSUBLOTE']    = isset($row['NRSUBLOTE'])    && $row['NRSUBLOTE']    ? $row['NRSUBLOTE']    : ' ';   
    }
    
    public function deletaTodosServicos($row){
        $query = "DELETE FROM RETIRADAGERAL WHERE CDFILIAL = :CDFILIAL AND CDPRODUTO = :CDPRODUTO AND DTRETGERAL = :DTRETGERAL AND ( CDALMOXARIFE = :CDALMOXARIFE OR ( :CDALMOXARIFE = ' ' AND CDALMOXARIFE IS NULL ) ) AND ( CDLOCALESTOQ = :CDLOCALESTOQ OR ( :CDLOCALESTOQ = ' ' AND CDLOCALESTOQ IS NULL ) ) AND ( NRLOTEESTQ = :NRLOTEESTQ OR ( :NRLOTEESTQ = ' ' AND NRLOTEESTQ IS NULL ) ) AND ( NRSUBLOTE = :NRSUBLOTE OR ( :NRSUBLOTE = ' ' AND NRSUBLOTE IS NULL ) )";
        $stmt = $this->connection->prepare($query);
        $this->setParamsEstoq($row);
        $params = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "CDPRODUTO"  => $row['CDPRODUTO'],
            "DTRETGERAL" => $row['DTRETGERAL'],
            "CDALMOXARIFE" => $row['CDALMOXARIFE'],
            "CDLOCALESTOQ" => $row['CDLOCALESTOQ'],
            "NRLOTEESTQ" => $row['NRLOTEESTQ'],
            "NRSUBLOTE" => $row['NRSUBLOTE']
            
        );
        $stmt->execute($params);
    }
    
    /* ROTINA  DE EXCLUSAO DE SERVICOS*/
    public function deletaServicos(Request $request, Response $response) {
        try {
            $servicos = $request->getDataSet()->getRows();
            $this->connection->beginTransaction();
            if (isset($servicos[0]) && isset($servicos[0]['deleteAllServices']) && $servicos[0]['deleteAllServices'] == true) {
                $this->deletaTodosServicos($servicos[0]);
            } else {
                foreach ($servicos as $servico) {
                    $query = "DELETE FROM RETIRADAGERAL WHERE CDFILIAL = :CDFILIAL AND CDPRODUTO = :CDPRODUTO AND DTRETGERAL = :DTRETGERAL AND NRRETGERAL = :NRRETGERAL AND CDSERVFILI = :CDSERVFILI AND ( CDALMOXARIFE = :CDALMOXARIFE OR ( :CDALMOXARIFE = ' ' AND CDALMOXARIFE IS NULL ) ) AND ( CDLOCALESTOQ = :CDLOCALESTOQ OR ( :CDLOCALESTOQ = ' ' AND CDLOCALESTOQ IS NULL ) ) AND ( NRLOTEESTQ = :NRLOTEESTQ OR ( :NRLOTEESTQ = ' ' AND NRLOTEESTQ IS NULL ) ) AND ( NRSUBLOTE = :NRSUBLOTE OR ( :NRSUBLOTE = ' ' AND NRSUBLOTE IS NULL ) )";
                    $stmt = $this->connection->prepare($query);
                    $params = array(
                        "CDFILIAL"     => $servico['CDFILIAL'],
                        "CDPRODUTO"    => $servico['CDPRODUTO'],
                        "DTRETGERAL"   => $servico['DTRETGERAL'],
                        "CDSERVFILI"   => $servico['CDSERVFILI'],
                        "NRRETGERAL"   => $servico['NRRETGERAL'],
                        "CDALMOXARIFE" => $servico['CDALMOXARIFE'],
                        "CDLOCALESTOQ" => $servico['CDLOCALESTOQ'],
                        "NRLOTEESTQ"   => $servico['NRLOTEESTQ'],
                        "NRSUBLOTE"    => $servico['NRSUBLOTE']
                    );
                    $stmt->execute($params);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e){
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    /* FIM DA ROTINA  DE EXCLUSAO DE SERVICOS*/
    
    /* ROTINA  DE EXCLUSAO DE PRODUTOS*/
    public function deletaProdutosRetirada(Request $request, Response $response) {
        $produtos = $request->getDataSet()->getRows();
        foreach ($produtos as $produto) {
            try {
                $query = "DELETE FROM RETIRADAGERAL R WHERE R.CDFILIAL = :CDFILIAL AND R.CDPRODUTO = :CDPRODUTO AND R.DTRETGERAL = :DTRETGERAL AND (R.CDALMOXARIFE = :CDALMOXARIFE OR (:CDALMOXARIFE = ' ' AND R.CDALMOXARIFE IS NULL)) AND (R.CDLOCALESTOQ = :CDLOCALESTOQ OR (:CDLOCALESTOQ = ' ' AND R.CDLOCALESTOQ IS NULL)) AND (R.NRLOTEESTQ = :NRLOTEESTQ OR (:NRLOTEESTQ = ' ' AND R.NRLOTEESTQ IS NULL)) AND (R.NRSUBLOTE = :NRSUBLOTE OR (:NRSUBLOTE = ' ' AND R.NRSUBLOTE IS NULL)) AND EXISTS (SELECT 1 FROM SERVICO S WHERE S.CDFILIAL = R.CDFILIAL AND S.CDSERVFILI = R.CDSERVFILI AND ((INSTR(:P_SERVSEL, S.CDSERVFILI) > 0) OR (:P_SERVSEL = 'T')) AND ((INSTR(:P_GRUPSEL, S.CDGRUPSERV) > 0) OR (:P_GRUPSEL = 'T')))";
                $stmt = $this->connection->prepare($query);
                $params = array(
                    "CDFILIAL"   => $produto['CDFILIAL'],
                    "CDPRODUTO"  => $produto['CDPRODUTO'],
                    "DTRETGERAL" => $produto['DTRETGERAL'],
                    "CDALMOXARIFE" => $produto['CDALMOXARIFE'],
                    "CDLOCALESTOQ" => $produto['CDLOCALESTOQ'],
                    "NRLOTEESTQ" => $produto['NRLOTEESTQ'],
                    "NRSUBLOTE" => $produto['NRSUBLOTE'],
                    "P_SERVSEL" => $produto['SERVSEL'],
                    "P_GRUPSEL" => $produto['GRUPSEL']
                );
                $stmt->execute($params);
    	    } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
    	    }
        }
    }
    /* FIM DA ROTINA  DE EXCLUSAO DE PRODUTOS*/

    /* FUNCOES GERAIS */
    public function validaQuantidades($row, $response) {
        if($row['QTRETIRADA'] > 0) {
            if(($row['QTDEVOLVRETI']) > ($row['QTEXTRARETI'] + $row['QTPREVISRETI'])) {
                throw new \Exception("Quantidade devolvida não pode ser maior que a quantidade requisitada.");
            }
            
            if($row['QTRETIRADA'] > $row['QTPREVISRETI']) {
                $row['QTEXTRARETI'] = $row['QTRETIRADA'] - $row['QTPREVISRETI'];
                $row['QTDEVOLVRETI'] = 0;
            } else if($row['QTRETIRADA'] < $row['QTPREVISRETI']) {
                $row['QTDEVOLVRETI'] = $row['QTPREVISRETI'] - $row['QTRETIRADA'];
                $row['QTEXTRARETI'] = 0;
            }
        } else {
            if($row['QTEXTRARETI'] + $row['QTPREVISRETI'] <= 0) {
                throw new \Exception("Quantidade consumida deve ser informada.");
            }
            $row['QTRETIRADA'] = $row['QTPREVISRETI'] + $row['QTEXTRARETI'] - $row['QTDEVOLVRETI'];
        }
    }
    
    public function getProduct ($params){
        $query = "SELECT CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, TO_NUMBER(NVL(CDTIPCUSTEST, '3')), TO_DATE(:P_DTPOSICAO), SYSDATE, :P_CDALMOXARIFE,:P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE) VRCUSTPROD FROM PARAMFILIAL WHERE CDFILIAL = :P_CDFILIAL";
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        $result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(count($result) > 0)
            return $result[0];
        else
            return ['VRCUSTPROD' => 0];
    }
    
    public function verificaEstoqueNegativo($row, $response, &$errorProducts = null, &$errorMsg = null) {
        $cdfilial = isset($row['CDFILIAL_FILTRADA']) ? $row['CDFILIAL_FILTRADA'] : $row['CDFILIAL'];
        $query = "SELECT P.CDPRODUTO, P.NMPRODUTO, P.CDARVPROD, NVL(P.VRFATOCONV, 1) VRFATOCONV, F.CDFILIAL FROM PRODUTO P INNER JOIN PRODFILI F ON F.CDPRODUTO = P.CDPRODUTO AND F.CDFILIAL = :P_CDFILIAL WHERE P.CDPRODUTO = :P_CDPRODUTO";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "P_CDFILIAL"      => $cdfilial,
            "P_CDPRODUTO"     => $row['CDPRODUTO']
        );
        $stmt->execute($params);
        $dadosProduto = $stmt->fetch(\PDO::FETCH_ASSOC);
        if(!$dadosProduto){
            throw new \Exception("Produto não cadastrado para unidade ou inativo.");
        }
        $VRFATOCONV = ($dadosProduto['VRFATOCONV'] <= 0) ? 1 :  $dadosProduto['VRFATOCONV'];
        
        $query = "SELECT CDTIPCUSTEST, IDPERMESTNEG, IDPERMVISPOSEST FROM PARAMFILIAL WHERE CDFILIAL = :P_CDFILIAL";
        $stmt = $this->connection->prepare($query);
        $paramsParamFilial = array(
            "P_CDFILIAL"      => $cdfilial
        );
        $stmt->execute($paramsParamFilial);
        $paramFilial = $stmt->fetch(\PDO::FETCH_ASSOC);
        
        $CDTIPCUSTEST = "3";
        $IDPERMESTNEG = "N";
        
        if($paramFilial){
            $CDTIPCUSTEST = $paramFilial['CDTIPCUSTEST'];
            $IDPERMESTNEG = $paramFilial['IDPERMESTNEG'];
        }
        
        $query = "SELECT TRUNC((D.QTESTOQDIA / :P_VRFATOCONV), 5) AS QTESTOQDIA, CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, TO_NUMBER(:P_CDTIPCUSTEST), TO_DATE(:P_DTPOSICAO), SYSDATE, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE) VRCUSTPROD FROM POSESTDIA D WHERE ( D.CDFILIAL = :P_CDFILIAL ) AND ( D.CDALMOXARIFE = :P_CDALMOXARIFE ) AND ( D.CDLOCALESTOQ = :P_CDLOCALESTOQ ) AND ( D.NRLOTEESTQ = :P_NRLOTEESTQ ) AND ( D.NRSUBLOTE = :P_NRSUBLOTE ) AND ( D.CDPRODUTO = :P_CDPRODUTO ) AND ( D.DTPOSIESTQ = DTPOSICAO_POSESTDIA(:P_CDFILIAL, :P_CDPRODUTO, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE, TO_DATE(:P_DTPOSICAO) + 1) )";
        $stmt = $this->connection->prepare($query);
        $cdalmoxarife = isset($row['CDALMOXARIFE']) ? $row['CDALMOXARIFE'] : " ";
        $cdlocalestoq = isset($row['CDLOCALESTOQ']) ? $row['CDLOCALESTOQ'] : " ";
        $nrloteestq = isset($row['NRLOTEESTQ']) ? $row['NRLOTEESTQ'] : " ";
        $nrsublote = isset($row['NRSUBLOTE']) ? $row['NRSUBLOTE'] : " ";
        
        $params = array(
            "P_CDFILIAL"      => $cdfilial,
            "P_CDPRODUTO"     => $row['CDPRODUTO'],
            "P_CDALMOXARIFE"  => $cdalmoxarife,
            "P_CDLOCALESTOQ"  => $cdlocalestoq,
            "P_DTPOSICAO"     => $row['DTRETGERAL'],
            "P_NRLOTEESTQ"    => $nrloteestq,
            "P_NRSUBLOTE"     => $nrsublote,
            "P_VRFATOCONV"    => $VRFATOCONV,
            "P_CDTIPCUSTEST"  => $CDTIPCUSTEST
        );
        
        $stmt->execute($params);
        $estoque = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        $row['VRUNILANCTO'] = isset($estoque[0]['VRCUSTPROD']) ? $estoque[0]['VRCUSTPROD'] : 0;

        if(isset($estoque[0]['QTESTOQDIA'])){
            $qtestoqdia = $estoque[0]['QTESTOQDIA'];
        }else{
            $qtestoqdia = 0;
        }        
        if((!isset($row['NEWPRODUCT']) && isset($row['QTRETIRADA']) && $IDPERMESTNEG === 'S') || (!isset($row['NEWPRODUCT']) && $qtestoqdia >= $row['QTRETIRADA'])){
            return true;
        }
        $cdarvprod = $dadosProduto['CDARVPROD'];
        $nmproduto = $dadosProduto['NMPRODUTO'];
       
        /*  VALIDAÇÃO PARA EXIBIÇÃO DE SALDO EM ESTOQUE (QTESTOQDIA) ESTÁ UTILIZANDO APENAS O PARAMETRO IDEXIBESALDOESTRETI, 
            O PARAMETRO IDPERMVISPOSEST FOI IGNORADO NO MOBILE;
        if(!isset($row['NEWPRODUCT']) && isset($row['QTRETIRADA']) && ($estoque[0]['IDPERMESTNEG'] === 'S' || !isset($row['NEWPRODUCT']) && $qtestoqdia >= $row['QTRETIRADA'])){
        if($estoque[0]['IDPERMVISPOSEST'] === 'S') {
            if($errorMsg !== null)
                $errorMsg .= $cdarvprod.' - '.$nmproduto.' - '.$qtestoqdia.';<br>';
            if($errorProducts !== null)
                array_push($errorProducts, array( "CDPRODUTO" => $row['CDPRODUTO'], "CDARVPROD" => $cdarvprod, "QTESTOQDIA" => $qtestoqdia ));
        } else {
            if($errorMsg !== null)
                $errorMsg .= $cdarvprod.' - '.$nmproduto.';<br>';
            if($errorProducts !== null)
                array_push($errorProducts, array( "CDPRODUTO" => $row['CDPRODUTO'], "CDARVPROD" => $cdarvprod ));
        }*/

        if(isset($row['NEWPRODUCT']) && $errorMsg !== null){
            if($qtestoqdia === 0){
                $errorMsg .= 'Produto informado não possui saldo em estoque! <br>';
            }else if($qtestoqdia < $row['QTRETIRADA'] && $row['IDEXIBESALDOESTRETI'] === 'S'){
                $errorMsg .= 'A quantidade consumida informada para o produto é maior do que o saldo ('.$qtestoqdia.') disponível no estoque! <br>';
            }else if($qtestoqdia < $row['QTRETIRADA'] && $row['IDEXIBESALDOESTRETI'] === 'N'){
                $errorMsg .= 'A quantidade consumida informada para o produto é maior do que o saldo disponível no estoque! <br>';
            }
        }
        
        if(!isset($row['NEWPRODUCT']) && $errorMsg !== null )
            $errorMsg .= $cdarvprod.' - '.$nmproduto.' - '.$qtestoqdia.';<br>';
            
        if($errorProducts !== null)
            array_push($errorProducts, array( "CDPRODUTO" => $row['CDPRODUTO'], "CDARVPROD" => $cdarvprod, "QTESTOQDIA" => $qtestoqdia, "NMPRODUTO" => $nmproduto, "CDALMOXARIFE" => $cdalmoxarife, "CDLOCALESTOQ" => $cdlocalestoq, "NRLOTEESTQ" => $nrloteestq, "NRSUBLOTE" => $nrsublote));
        
        return false;
    }
    
    public function updataRetiradaGeral($servico, $produto, $response, $dataRetirada) {
        $query = "UPDATE RETIRADAGERAL SET QTCONRETIGER = :QTCONRETIGER, QTEXTRETIGER = :QTEXTRETIGER, QTDEVRETIGER = :QTDEVRETIGER, VRUNIRETGER = :VRUNILANCTO, CDALMOXARIFE = :CDALMOXARIFE, CDLOCALESTOQ = :CDLOCALESTOQ, NRLOTEESTQ = :NRLOTEESTQ, NRSUBLOTE = :NRSUBLOTE WHERE NRRETGERAL = :NRRETGERAL AND CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND DTRETGERAL = :DTRETGERAL AND CDPRODUTO = :CDPRODUTO AND NVL(CDALMOXARIFE, ' ') = :CDALMOXARIFE_BACKUP AND NVL(CDLOCALESTOQ, ' ') = :CDLOCALESTOQ_BACKUP AND NVL(NRLOTEESTQ, ' ') = :NRLOTEESTQ_BACKUP AND NVL(NRSUBLOTE, ' ') = :NRSUBLOTE_BACKUP";
        $stmt = $this->connection->prepare($query);
        $CDALMOXARIFE = isset($produto['CDALMOXARIFE']) ? $produto['CDALMOXARIFE'] : ' ';
        $CDLOCALESTOQ = isset($produto['CDLOCALESTOQ']) ? $produto['CDLOCALESTOQ'] : ' ';
        $NRLOTEESTQ = isset($produto['NRLOTEESTQ']) ? $produto['NRLOTEESTQ'] : ' ';
        $NRSUBLOTE = isset($produto['NRSUBLOTE']) ? $produto['NRSUBLOTE'] : ' ';
        $CDALMOXARIFE_BACKUP = isset($produto['CDALMOXARIFE_BACKUP']) ? $produto['CDALMOXARIFE_BACKUP'] : ' ';
        $CDLOCALESTOQ_BACKUP = isset($produto['CDLOCALESTOQ_BACKUP']) ? $produto['CDLOCALESTOQ_BACKUP'] : ' ';
        $NRLOTEESTQ_BACKUP = isset($produto['NRLOTEESTQ_BACKUP']) ? $produto['NRLOTEESTQ_BACKUP'] : ' ';
        $NRSUBLOTE_BACKUP = isset($produto['NRSUBLOTE_BACKUP']) ? $produto['NRSUBLOTE_BACKUP'] : ' ';
        $params = array(
            "CDFILIAL"             => $servico['CDFILICPLA'],
            "CDSERVFILI"           => $servico['CDSERVCPLA'],
            "NRRETGERAL"           => $servico['NRRETGERAL'],
            "DTRETGERAL"           => $dataRetirada,
            "CDPRODUTO"            => $produto['CDPRODUTO'],
            "QTEXTRETIGER"         => $servico['QTEXTRARETI'],
            "QTDEVRETIGER"         => $servico['QTDEVOLVRETI'],
            "QTCONRETIGER"         => $servico['QTRETIRADA'],
            "VRUNILANCTO"          => $produto['VRUNILANCTO'],
            "CDALMOXARIFE"         => $CDALMOXARIFE,
            "CDLOCALESTOQ"         => $CDLOCALESTOQ,
            "NRLOTEESTQ"           => $NRLOTEESTQ,
            "NRSUBLOTE"            => $NRSUBLOTE,
            "CDALMOXARIFE_BACKUP"  => $CDALMOXARIFE_BACKUP,
            "CDLOCALESTOQ_BACKUP"  => $CDLOCALESTOQ_BACKUP,
            "NRLOTEESTQ_BACKUP"    => $NRLOTEESTQ_BACKUP,
            "NRSUBLOTE_BACKUP"     => $NRSUBLOTE_BACKUP
        );
        //Obtém o custo unitário atualizado
        $cdalmoxarife = $CDALMOXARIFE;
        $cdlocalestoq = $CDLOCALESTOQ;
        $nrloteestq = $NRLOTEESTQ;
        $nrsublote = $NRSUBLOTE;
        $cdfilial = $servico['CDFILICPLA'];
        $paramsCusto = array(
            "P_CDFILIAL"      => $cdfilial,
            "P_CDPRODUTO"     => $produto['CDPRODUTO'],
            "P_CDALMOXARIFE"  => $cdalmoxarife,
            "P_CDLOCALESTOQ"  => $cdlocalestoq,
            "P_DTPOSICAO"     => $dataRetirada,
            "P_NRLOTEESTQ"    => $nrloteestq,
            "P_NRSUBLOTE"     => $nrsublote
        );
        $params['VRUNILANCTO'] = $this->getProduct($paramsCusto)['VRCUSTPROD'];
        
        //coleta de dados
        $this->libUtilities->saveCollectedData($servico['CDFILICPLA'],'RETIRADA');
        $stmt->execute($params);
    }
    /* FIM DAS FUNCOES GERAIS */
    
    /*PROCEDURES*/
    public function executaProcedureConsumoPlanejado($row, $response, $request, $lista) {
        $consplan   = $this->geraNovoCodigo('CONSPLAN'.$row['CDFILIAL']);
        $nrconsplan = $this->validaNewCodeConsplan($consplan);
        
        $procedureConsumoPlanejado = new StoredProcedure($this->connection, 'CONSUMO_PLANEJADO_PLA');
        $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $lista,             Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTRETGERAL'], Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTRETGERAL'], Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "A",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, $row['CONSUMO'],    Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "P",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                  Param::PARAM_TYPE_INT));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $nrconsplan,        Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, "0",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, "zzzzzzzzzz",       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'], Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "",                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "",                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->execute();
        
        return $nrconsplan;
    }
    
    public function executaProcedureRequisicaoAnalitica($row, $nrconsplan, $lista) {
        $cdoperador = $this->environment->getCdOperador();
        $procedure = new StoredProcedure($this->connection, 'REQUISICAO_ANALITICA_SETOR_PLA');
        // P_FILIAL passa sempre T para que a procedure considere os servicos da lista
        $procedure->addParam(new Param('P_FILIAL',        Param::PARAM_INPUT, "T",     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SERVICOS',      Param::PARAM_INPUT, $lista,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL',     Param::PARAM_INPUT, $row['DTRETGERAL'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL',       Param::PARAM_INPUT, $row['DTRETGERAL'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDARQUIVO',     Param::PARAM_INPUT, "A",                  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPRODUTOS',    Param::PARAM_INPUT, $row['CONSUMO'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSEPPRATO',    Param::PARAM_INPUT, "P",                  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PEACRESCIMO',   Param::PARAM_INPUT, 0,                    Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_IDARREDOND',    Param::PARAM_INPUT, "N",                  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRCONSPLAN',    Param::PARAM_INPUT, $nrconsplan,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTOINI',  Param::PARAM_INPUT, "0",                  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTOFIN',  Param::PARAM_INPUT, "zzzzzzzzzz",         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',    Param::PARAM_INPUT, $cdoperador,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTHRCALCPR',    Param::PARAM_INPUT, $row['DTHRCALCPR'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_STDTINICIAL',   Param::PARAM_INPUT, "",                   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_STDTFINAL',     Param::PARAM_INPUT, "",                   Param::PARAM_TYPE_STR));

        $procedure->execute();
    }
    /*FIM DAS PROCEDURES*/
    
    public function verificaPermissaoData(Request $request, Response $response) {
        $row = $request->getRow();
        $dataSet = array();
        $queryDataFilial = "SELECT DTPROCESSA, IDLANCDTMAIFIL, NRDIASSAIDPDFIL, IDLANCDTMENFIL, NRDIASSAIDFIL, DTPROCESSA FROM PARAMFILIAL WHERE CDFILIAL = :P_CDFILIAL";
        $params = array(
            "P_CDFILIAL" => $row["CDFILIAL"]
        );
        $dtPermissaoFilial = $this->connection->fetchAll($queryDataFilial, $params);
        if($dtPermissaoFilial && sizeof($dtPermissaoFilial)) {
            if($dtPermissaoFilial[0]["IDLANCDTMAIFIL"] === "S")
                $dataSet["NRDIASSAIDPD"] = $dtPermissaoFilial[0]["NRDIASSAIDPDFIL"] ? $dtPermissaoFilial[0]["NRDIASSAIDPDFIL"] : 0;
                
            if($dtPermissaoFilial[0]["IDLANCDTMENFIL"] === "S")
                $dataSet["NRDIASSAID"] = $dtPermissaoFilial[0]["NRDIASSAIDFIL"] ? $dtPermissaoFilial[0]["NRDIASSAIDFIL"] : 0;
        }
        if(!isset($dataSet["NRDIASSAIDPD"]) || !isset($dataSet["NRDIASSAID"])) {
            $queryDataGeral = "SELECT IDLANCDTMAI, NRDIASSAIDPD, IDLANCDTMEN, NRDIASSAID FROM PARAMESTOQUE";
            $dtPermissaoGeral = $this->connection->fetchAll($queryDataGeral);

            if(!isset($dataSet["NRDIASSAIDPD"]) && $dtPermissaoGeral[0]["IDLANCDTMAI"] === "S")
                $dataSet["NRDIASSAIDPD"] = $dtPermissaoGeral[0]["NRDIASSAIDPD"] ? $dtPermissaoGeral[0]["NRDIASSAIDPD"] : 0;
                
            if(!isset($dataSet["NRDIASSAID"]) && $dtPermissaoGeral[0]["IDLANCDTMEN"] === "S")
                $dataSet["NRDIASSAID"] = $dtPermissaoGeral[0]["NRDIASSAID"] ? $dtPermissaoGeral[0]["NRDIASSAID"] : 0;
        }
        if(!isset($dataSet["NRDIASSAIDPD"]) || !isset($dataSet["NRDIASSAID"])){
            $dataSet["NRDIASSAID"] = 0;
            $dataSet["NRDIASSAIDPD"] = 0;
        }
        $dataSet["DTPROCESSA"] = $dtPermissaoFilial[0]["DTPROCESSA"];
        $response->addDataSet(new DataSet("diasPermissaoRetirada", $dataSet));
    }
    
    public function verificaParamExibeSaldoControlAlmox (Request $request, Response $response) { 
        $row = $request->getRow();
        $query= "SELECT CASE WHEN PF.CDFILIAL IS NULL THEN 'N' ELSE 'S' END EXISTE_PARAMFILIAL, CASE WHEN PF.CDFILIAL IS NULL THEN 'N' ELSE 'S' END EXISTE_PARAMPLANEJA, PP.CDFILIAL AS CDFILIAL_FILTRADA, PP.IDEXIBESALDOESTRETI, PF.IDUTILALMOX, PF.IDUTILLOTE, PF.IDCTRLESTQ FROM FILIAL F LEFT JOIN PARAMPLANEJA PP ON PP.CDFILIAL = F.CDFILIAL LEFT JOIN PARAMFILIAL PF ON PF.CDFILIAL = F.CDFILIAL WHERE F.CDFILIAL =:P_CDFILIAL";
        $params = array(
            "P_CDFILIAL" => $row["CDFILIAL"]
        );
        $result = $this->connection->fetchAll($query, $params);
        $dataSet = $result;
        $response->addDataSet(new DataSet("data", $dataSet));
        
    }
    
    public function checkDuplicateRecord($row, $response) {
        $query = "SELECT
                        R.NRRETGERAL
                    FROM
                        RETIRADAGERAL R
                    WHERE
                        R.CDPRODUTO        = :CDPRODUTO
                        AND R.DTRETGERAL   = :DTRETGERAL
                        AND R.CDFILIAL     = :CDFILIAL
                        AND R.CDALMOXARIFE = :CDALMOXARIFE
                        AND R.CDLOCALESTOQ = :CDLOCALESTOQ
                        AND R.NRLOTEESTQ   = :NRLOTEESTQ
                        AND EXISTS (SELECT 1 FROM SERVICO S WHERE S.CDFILIAL = R.CDFILIAL AND S.CDSERVFILI = R.CDSERVFILI 
                                                             AND ((INSTR(:P_SERVSEL, S.CDSERVFILI) > 0) OR (:P_SERVSEL = 'T'))
                                                             AND ((INSTR(:P_GRUPSEL, S.CDGRUPSERV) > 0) OR (:P_GRUPSEL = 'T')) )";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDPRODUTO"    => $row['CDPRODUTO'],
            "DTRETGERAL"   => $row['DTRETGERAL'],
            "CDFILIAL"     => $row['CDFILIAL_FILTRADA'],
            "CDALMOXARIFE" => $row['CDALMOXARIFE'],
            "CDLOCALESTOQ" => $row['CDLOCALESTOQ'],
            "NRLOTEESTQ"   => $row['NRLOTEESTQ'],
            "P_SERVSEL" => $row['SERVSEL'],
            "P_GRUPSEL" => $row['GRUPSEL']
        );
        $stmt->execute($params);
        $produtoAlmoxarife = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(!empty($produtoAlmoxarife) && isset($row['NEWPRODUCT']) && $row['NEWPRODUCT']) {
            throw new \Exception("Produto já cadastrado para este almoxarifado/localização/lote.");
        } else if(!empty($produtoAlmoxarife) && isset($row['UPDATEPRODUCT']) && $row['UPDATEPRODUCT']){
            throw new \Exception($row['CDARVPROD'] . " - " . $row['NMPRODUTO'] . " foi cadastrado mais de uma vez para o mesmo almoxarifado/localização/lote.");
        }
    }
    
    private function geraNovoCodigo($contador){
        return $this->newCode->getCodeByTableName($contador, $this->environment->getNrOrg());
    }
    
}