<?php
namespace Teknisa\PLA\Modules\Mymeals\Services;

use Doctrine\ORM\EntityManager;
use Planejamento\Entities\Confirmarefe;
use Planejamento\Entities\Compcard;
use Planejamento\Entities\Cardacli;
use Planejamento\Entities\Cardadia;
use Zeedhi\Framework\DataSource;

class Pla02010AlteracaoComensaisService {

    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;

    public function __construct(EntityManager $entityManager, $environment, DataSource\Manager $dataSourceManager)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function iniciaTransacao(){
        $this->entityManager->beginTransaction();
    }

    public function commit(){
        $this->entityManager->commit();
    }

    public function flush(){
        $this->entityManager->flush();
    }

    public function rollback(){
        $this->entityManager->rollback();
    }

    public function atualizaCardacli($row){
        $cardacli = $this->entityManager->getRepository('\Planejamento\Entities\Cardacli')
            ->findOneBy(
                array(
                    "nrorg" => $row['NRORG'],
                    "nrcardapio" => $row['NRCARDAPIO'],
                    "cdcliente" => $row['CDCLIENTE'],
                    "cddestfili" => $row['CDDESTFILI']
                )
            );
            
        if ($cardacli instanceof Cardacli) {
            $cardacli->setQtcomesticli($row['QTCOMESTICLI']);
            $cardacli->setNrorgultatu($row['NRORG']);
            $cardacli->setDtultatu(new \DateTime());
            $cardacli->setCdoperultatu($row['CDOPERADOR']);
            
            $this->entityManager->persist($cardacli);
        }
    }
    
    public function atualizaCompcard($row){
        $compcard = $this->entityManager->getRepository('\Planejamento\Entities\Compcard')
            ->findOneBy(
                array(
                    "nrorg" => $row['NRORG'],
                    "cdfilial" => $row['CDFILIAL'],
                    "nrcardapio" => $row['NRCARDAPIO'],
                    "nrseqpratcar" => $row['NRSEQPRATCAR']
                )
            );
            
        if ($compcard instanceof Compcard) {
            $compcard->setVrpeconscard($row['VRPECONSCARD']);
            $compcard->setNrorgultatu($row['NRORG']);
            $compcard->setDtultatu(new \DateTime());
            $compcard->setCdoperultatu($row['CDOPERADOR']);
            
            $this->entityManager->persist($compcard);
        }
    }

    public function atualizaCardadia($row){
        $cardadia = $this->entityManager->getRepository('\Planejamento\Entities\Cardadia')
            ->findOneBy(
                array(
                    "nrorg" => $row['NRORG'],
                    "cdfilial" => $row['CDFILIAL'],
                    "nrcardapio" => $row['NRCARDAPIO']
                )
            );
                
        if($cardadia instanceof Cardadia){
            $cardadia->setNrorg($row['NRORG']);
            $cardadia->setCdfilial($row['CDFILIAL']);
            $cardadia->setNrcardapio($row['NRCARDAPIO']);
            $cardadia->setQtcomesticar($row['QTCOMESTICAR']);
            $cardadia->setQtcliesticar($row['QTCLIESTICAR']);
            $cardadia->setQtfunesticar($row['QTFUNESTICAR']);
            $cardadia->setQtdoaesticar($row['QTDOAESTICAR']);
            $cardadia->setQtestiturno1($row['QTESTITURNO1']);
            $cardadia->setQtestiturno2($row['QTESTITURNO2']);
            $cardadia->setQtestiturno3($row['QTESTITURNO3']);
            $cardadia->setQtestiturno4($row['QTESTITURNO4']);
                
            $this->entityManager->persist($cardadia);
        }
    }

    public function insereConfirmarefe($row){
        $datetime = new \DateTime();
        
        $maxconfirmarefe = $this->entityManager->getRepository('\Planejamento\Entities\Confirmarefe')
        ->findOneBy(array('nrorg' => $row['NRORG']), array('nrconfirmarefe' => "DESC"));
        
        if($maxconfirmarefe instanceof Confirmarefe){
            $maxconfirmarefe = $maxconfirmarefe->getNrconfirmarefe()+1;
        }else{
            $maxconfirmarefe = 1;
        }      
        
        $confirmarefe = new Confirmarefe();
        
        $confirmarefe->setNrconfirmarefe($maxconfirmarefe);
        $confirmarefe->setCdfilial($row['CDFILIAL']);
        $confirmarefe->setCdservfili($row['CDSERVFILI']);
        $confirmarefe->setNrcardapio($row['NRCARDAPIO']);
        $confirmarefe->setDtcardapio($datetime->createFromFormat('d/m/Y', $row['DTCARDAPIO']));
        $confirmarefe->setCdcliente($row['CDCLIENTE']);
        $confirmarefe->setCddestfili($row['CDDESTFILI']);
        $confirmarefe->setCdturno($row['CDTURNO']);
        $confirmarefe->setQtcomeoriginal($row['QTCLIESTIDIA']);
        $confirmarefe->setQtcomealterada($row['QTCLIESTICAR']);
        $confirmarefe->setQttotalalterada($row['QTCOMESTICAR']);
        $confirmarefe->setIdqtdeaprovada($row['IDQTDEAPROVADA']);
        $confirmarefe->setDsparametros($row['DSPARAMETROS']);
        $confirmarefe->setNrorg($row['NRORG']);
        $confirmarefe->setNrorginclusao($row['NRORG']);
        $confirmarefe->setNrorgultatu($row['NRORG']);
        $confirmarefe->setDtinclusao(new \DateTime());
        $confirmarefe->setDtinclusao(new \DateTime());
        $confirmarefe->setDtultatu(new \DateTime());
        $confirmarefe->setCdoperinclusao($row['CDOPERADOR']);
        $confirmarefe->setCdoperultatu($row['CDOPERADOR']);
        $this->entityManager->persist($confirmarefe);
        
    }
}